package org.bidib.wizard.comm;

import java.io.IOException;
import java.util.Date;
import java.util.concurrent.Semaphore;

import org.bidib.jbidibc.Bidib;
import org.bidib.jbidibc.MessageReceiver;
import org.bidib.wizard.comm.bidib.BidibCommunication;
import org.bidib.wizard.comm.mock.MockCommunication;
import org.bidib.wizard.mvc.preferences.model.Preferences;
import org.bidib.wizard.mvc.preferences.model.listener.PreferencesListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CommunicationFactory {
    private static final Logger LOGGER = LoggerFactory.getLogger(CommunicationFactory.class.getName());

    private static Communication instance = null;

    private static Semaphore semaphore = new Semaphore(1);

    static {
        Preferences.getInstance().addPreferencesListener(new PreferencesListener() {
            @Override
            public void commPortChanged(String commPort) {
                // try {
                // semaphore.acquire();
                // instance = null;
                // } catch (InterruptedException e) {
                // throw new RuntimeException(e);
                // } finally {
                // semaphore.release();
                // }
            }

            @Override
            public void mockChanged(boolean mock) {
                try {
                    semaphore.acquire();
                    LOGGER.debug("The mock has changed, mock: {}", mock);
                    if (mock) {
                        LOGGER.debug("Disable the message receiver.");
                        MessageReceiver.disable();
                        LOGGER.debug("Close the Bidib port.");
                        Bidib.close();
                    }
                    instance = null;
                }
                catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                catch (IOException e) {
                    LOGGER.warn("Close the bidib port failed.", e);
                    throw new RuntimeException(e);
                }
                finally {
                    semaphore.release();
                }
            }

            @Override
            public void startTimeChanged(Date startTime) {
            }

            @Override
            public void timeFactorChanged(int timeFactor) {
            }
        });
    }

    public static Communication getInstance() {
        if (instance == null) {
            try {
                semaphore.acquire();
                if (Preferences.getInstance().isMock()) {
                    instance = new MockCommunication();
                }
                else {
                    try {
                        Bidib.open(Preferences.getInstance().getCommPort());
                        instance = new BidibCommunication();
                    }
                    catch (Exception e) {
                        LOGGER.warn("Open Bidib failed.", e);
                    }
                }
            }
            catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
            finally {
                semaphore.release();
            }
        }
        return instance;
    }
}
