package org.bidib.wizard.comm.mock;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.BitSet;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.bidib.jbidibc.BidibLibrary;
import org.bidib.jbidibc.Feature;
import org.bidib.jbidibc.FirmwareUpdateStat;
import org.bidib.jbidibc.Node;
import org.bidib.jbidibc.ProtocolVersion;
import org.bidib.jbidibc.SoftwareVersion;
import org.bidib.jbidibc.enumeration.AddressTypeEnum;
import org.bidib.jbidibc.enumeration.FirmwareUpdateOperation;
import org.bidib.jbidibc.enumeration.FirmwareUpdateState;
import org.bidib.jbidibc.enumeration.IdentifyState;
import org.bidib.wizard.comm.AnalogPortStatus;
import org.bidib.wizard.comm.Communication;
import org.bidib.wizard.comm.CriticalFunctionStatus;
import org.bidib.wizard.comm.Direction;
import org.bidib.wizard.comm.FeedbackPortStatus;
import org.bidib.wizard.comm.FlagStatus;
import org.bidib.wizard.comm.InputPortStatus;
import org.bidib.wizard.comm.InputStatus;
import org.bidib.wizard.comm.LightPortStatus;
import org.bidib.wizard.comm.MacroStatus;
import org.bidib.wizard.comm.MotorPortStatus;
import org.bidib.wizard.comm.SoundPortStatus;
import org.bidib.wizard.comm.SpeedSteps;
import org.bidib.wizard.comm.SwitchPortStatus;
import org.bidib.wizard.mvc.main.model.Accessory;
import org.bidib.wizard.mvc.main.model.AddressData;
import org.bidib.wizard.mvc.main.model.AnalogPort;
import org.bidib.wizard.mvc.main.model.FeedbackPort;
import org.bidib.wizard.mvc.main.model.Flag;
import org.bidib.wizard.mvc.main.model.InputPort;
import org.bidib.wizard.mvc.main.model.LightPort;
import org.bidib.wizard.mvc.main.model.Macro;
import org.bidib.wizard.mvc.main.model.MotorPort;
import org.bidib.wizard.mvc.main.model.ServoPort;
import org.bidib.wizard.mvc.main.model.SoundPort;
import org.bidib.wizard.mvc.main.model.SwitchPort;
import org.bidib.wizard.mvc.main.model.TimeStartCondition;
import org.bidib.wizard.mvc.main.model.function.CriticalFunction;
import org.bidib.wizard.mvc.main.model.function.FlagFunction;
import org.bidib.wizard.mvc.main.model.function.Function;
import org.bidib.wizard.mvc.main.model.function.InputFunction;
import org.bidib.wizard.mvc.main.model.function.LightPortAction;
import org.bidib.wizard.mvc.main.model.function.MacroFunction;
import org.bidib.wizard.mvc.main.model.function.MotorPortAction;
import org.bidib.wizard.mvc.main.model.function.ServoPortAction;
import org.bidib.wizard.mvc.main.model.function.SwitchPortAction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MockCommunication implements Communication {
    private static final Logger LOGGER = LoggerFactory.getLogger(MockCommunication.class);

    private static final long UUID_ROOT = 0x02345678901233L;

    private static final long UUID_CHRISTOPH = 0x05345678901234L;

    private static final long UUID_WOLFGANG = 0x05345678901235L;

    private static final long UUID_ANDRE = 0x05345678901236L;

    private final Long[] uuids = new Long[] { UUID_CHRISTOPH, UUID_WOLFGANG, UUID_ANDRE };

    private final Map<Node, List<Accessory>> accessories = new HashMap<Node, List<Accessory>>();

    private final Map<Node, List<AnalogPort>> analogPorts = new HashMap<Node, List<AnalogPort>>();

    private final Map<Node, List<FeedbackPort>> feedbackPorts = new HashMap<Node, List<FeedbackPort>>();

    private final List<Flag> flags = new LinkedList<Flag>();

    private final Map<Node, List<InputPort>> inputPorts = new HashMap<Node, List<InputPort>>();

    private final Map<Node, List<LightPort>> lightPorts = new HashMap<Node, List<LightPort>>();

    private final Map<Node, List<Macro>> macros = new HashMap<Node, List<Macro>>();

    private final Map<Node, List<MotorPort>> motorPorts = new HashMap<Node, List<MotorPort>>();

    private final List<Node> nodes = new LinkedList<Node>();

    private final Map<Node, List<ServoPort>> servoPorts = new HashMap<Node, List<ServoPort>>();

    private final Map<Node, List<SoundPort>> soundPorts = new HashMap<Node, List<SoundPort>>();

    private final Map<Node, List<SwitchPort>> switchPorts = new HashMap<Node, List<SwitchPort>>();

    private final Map<Integer, Integer> cvList = new HashMap<Integer, Integer>();

    public MockCommunication() {
        org.bidib.wizard.mvc.main.model.Node node = initNode(new byte[] { 0 }, UUID_ROOT);

        nodes.add(node.getNode());

        byte localAddress = 1;

        for (long uuid : uuids) {
            node = initNode(new byte[] { localAddress++, 0 }, uuid);
            nodes.add(node.getNode());

            for (int id = 0; id < 5; id++) {
                AnalogPort port = new AnalogPort();

                port.setId(id);
                analogPorts.get(node.getNode()).add(port);
            }

            for (int id = 0; id < 32; id++) {
                FeedbackPort port = new FeedbackPort();
                Collection<AddressData> addresses = new LinkedList<AddressData>();
                AddressData addressData =
                    new AddressData((int) Math.round(Math.random() * 10000), AddressTypeEnum.LOCOMOTIVE_FORWARD);

                addressData.setSpeed((int) Math.round(Math.random() * 150));
                addresses.add(addressData);
                if (id % 2 == 0) {
                    addressData =
                        new AddressData((int) Math.round(Math.random() * 10000), AddressTypeEnum.LOCOMOTIVE_BACKWARD);
                    addressData.setSpeed((int) Math.round(Math.random() * 150));
                    addresses.add(addressData);
                }
                port.setAddresses(addresses);
                port.setFreeze((int) Math.round(Math.random() * 10) < 2);
                port.setId(id);
                port.setSignal((int) Math.round(Math.random() * 10) < 2);
                port.setStatus(id % 2 == 0 ? FeedbackPortStatus.OCCUPIED : FeedbackPortStatus.FREE);
                port.setValid((int) Math.round(Math.random() * 10) < 2);
                feedbackPorts.get(node.getNode()).add(port);
            }

            for (int id = 0; id < 16; id++) {
                Flag flag = new Flag();

                flag.setId(id);
                flags.add(flag);
            }

            for (int id = 0; id < 16; id++) {
                InputPort port = new InputPort();

                port.setId(id);
                port.setStatus(id % 2 == 0 ? InputPortStatus.ON : InputPortStatus.OFF);
                inputPorts.get(node.getNode()).add(port);
            }

            for (int id = 0; id < 10; id++) {
                Macro macro = new Macro(10);

                macro.setId(id);
                macros.get(node.getNode()).add(macro);
            }

            for (int id = 0; id < 24; id++) {
                Accessory accessory = new Accessory();

                accessory.setMacroSize(8);
                if (id < 10) {
                    for (int index = 0; index < macros.get(node.getNode()).size() && index < 4; index++) {
                        accessory.addMacro(macros.get(node.getNode()).get(
                            (int) (Math.random() * macros.get(node.getNode()).size())));
                    }
                }
                accessory.setId(id);
                accessories.get(node.getNode()).add(accessory);
            }

            if (uuid != UUID_WOLFGANG) {
                for (int id = 0; id < 20; id++) {
                    LightPort port = new LightPort();

                    port.setId(id);
                    lightPorts.get(node.getNode()).add(port);
                }
            }

            if (uuid != UUID_WOLFGANG) {
                for (int id = 0; id < 3; id++) {
                    MotorPort port = new MotorPort();

                    port.setId(id);
                    motorPorts.get(node.getNode()).add(port);
                }
                if (uuid == UUID_ANDRE) {
                    MotorPort port = new MotorPort();

                    port.setId(3);
                    motorPorts.get(node.getNode()).add(port);
                }
            }

            for (int id = 0; id < 4; id++) {
                ServoPort port = new ServoPort();

                port.setId(id);
                servoPorts.get(node.getNode()).add(port);
            }

            for (int id = 0; id < 2; id++) {
                SoundPort port = new SoundPort();

                port.setId(id);
                soundPorts.get(node.getNode()).add(port);
            }

            for (int id = 0; id < 15; id++) {
                SwitchPort port = new SwitchPort();

                port.setId(id);
                switchPorts.get(node.getNode()).add(port);
            }
        }

        Node bidibNode = nodes.get(1);
        Macro macro = macros.get(bidibNode).get(0);
        Collection<Function> functions = new LinkedList<Function>();

        functions.add(new LightPortAction(LightPortStatus.ON, 1, lightPorts.get(bidibNode).get(0)));
        functions.add(new InputFunction(InputStatus.QUERY1, inputPorts.get(bidibNode).get(0)));
        functions.add(new LightPortAction(LightPortStatus.OFF, 2, lightPorts.get(bidibNode).get(0)));
        functions.add(new MacroFunction(MacroStatus.START, macros.get(bidibNode).get(3).getId()));
        functions.add(new LightPortAction(LightPortStatus.ON, 2, lightPorts.get(bidibNode).get(1)));
        functions.add(new FlagFunction(FlagStatus.QUERY, flags.get(1)));
        functions.add(new FlagFunction(FlagStatus.CLEAR, flags.get(1)));
        functions.add(new LightPortAction(LightPortStatus.ON, 2, lightPorts.get(bidibNode).get(2)));
        functions.add(new MacroFunction(MacroStatus.START, macros.get(bidibNode).get(2).getId()));
        macro.clearFunctions();
        macro.addFunctionsAfter(macro.getFunctionCount() - 1, functions.toArray(new Function[0]));
        macro.setSpeed(128);

        Calendar time = Calendar.getInstance();

        time.set(Calendar.MINUTE, 0);

        TimeStartCondition startCondition = new TimeStartCondition(time);

        startCondition.setHourly(true);
        macro.addStartCondition(startCondition);

        macros.get(bidibNode).get(1).addStartCondition(new TimeStartCondition(time));

        bidibNode = nodes.get(2);
        macro = macros.get(bidibNode).get(0);
        functions = new LinkedList<Function>();
        functions.add(new MacroFunction(MacroStatus.START, macros.get(bidibNode).get(0).getId()));
        functions.add(new SwitchPortAction(SwitchPortStatus.ON, 0, switchPorts.get(bidibNode).get(0)));

        List<ServoPort> servoPorts = this.servoPorts.get(bidibNode);

        if (servoPorts != null && servoPorts.size() > 0) {
            functions.add(new ServoPortAction(0, servoPorts.get(0), 80));
        }

        List<MotorPort> motorPorts = this.motorPorts.get(bidibNode);

        if (motorPorts != null && motorPorts.size() > 2) {
            functions.add(new MotorPortAction(MotorPortStatus.FORWARD, 0, motorPorts.get(2), 80));
        }
        functions.add(new InputFunction(InputStatus.QUERY1, inputPorts.get(bidibNode).get(9)));
        functions.add(new FlagFunction(FlagStatus.SET, flags.get(1)));
        functions.add(new FlagFunction(FlagStatus.QUERY, flags.get(1)));

        List<LightPort> lightPorts = this.lightPorts.get(bidibNode);

        if (lightPorts != null && lightPorts.size() > 13) {
            functions.add(new LightPortAction(LightPortStatus.UP, 2, lightPorts.get(13)));
        }
        functions.add(new CriticalFunction(CriticalFunctionStatus.BEGIN));
        macro.clearFunctions();
        macro.addFunctionsAfter(macro.getFunctionCount() - 1, functions.toArray(new Function[0]));
    }

    @Override
    public List<Feature> getFeatures(Node node) {
        List<Feature> features = new ArrayList<Feature>();
        // return some dummy features
        features.add(new Feature(BidibLibrary.FEATURE_BST_VOLT_ADJUSTABLE, 1));
        features.add(new Feature(BidibLibrary.FEATURE_BST_CUTOUT_ON, 1));
        features.add(new Feature(BidibLibrary.FEATURE_BST_AMPERE_ADJUSTABLE, 1));
        features.add(new Feature(BidibLibrary.FEATURE_BST_CV_AVAILABLE, 1));
        return features;
    }

    @Override
    public void activateAnalogPort(Node node, int port, AnalogPortStatus status) {
        LOGGER.info("activate analog port " + port + " on node " + node + " with status " + status);
    }

    @Override
    public void activateLightPort(Node node, int port, LightPortStatus status) {
        LOGGER.info("activate light port " + port + " on node " + node + " with status " + status);
    }

    @Override
    public void activateMotorPort(Node node, int port, MotorPortStatus status) {
        LOGGER.info("activate motor port " + port + " on node " + node + " with status " + status);
    }

    @Override
    public void activateServoPort(Node node, int port, int value) {
        LOGGER.info("activate servo port " + port + " on node " + node + " with value " + value);
    }

    @Override
    public void activateSoundPort(Node node, int port, SoundPortStatus status) {
        LOGGER.info("activate sound port " + port + " on node " + node + " with status " + status);
    }

    @Override
    public void activateSwitchPort(Node node, int port, SwitchPortStatus status) {
        LOGGER.info("activate switch port " + port + " on node " + node + " with status " + status);
    }

    @Override
    public void boosterOff(Node node) {
        LOGGER.info("switch booster off on node " + node);
    }

    @Override
    public void boosterOn(Node node) {
        LOGGER.info("switch booster on on node " + node);
    }

    @Override
    public void boosterQuery(Node node) {
        LOGGER.info("query booster on node " + node);
    }

    @Override
    public Collection<Accessory> getAccessories(Node node) {
        return Collections.unmodifiableCollection(accessories.get(node));
    }

    @Override
    public int getAccessoryLength(Node node) {
        return 8;
    }

    @Override
    public Collection<AnalogPort> getAnalogPorts(Node node) {
        return Collections.unmodifiableCollection(analogPorts.get(node));
    }

    @Override
    public int getBoosterMaximumCurrent(Node node) {
        return 2000;
    }

    @Override
    public Collection<FeedbackPort> getFeedbackPorts(Node node) {
        return Collections.unmodifiableCollection(feedbackPorts.get(node));
    }

    @Override
    public void getFeedbackPortStatus(Node node, int begin, int end) {
    }

    @Override
    public Collection<Flag> getFlags() {
        return Collections.unmodifiableCollection(flags);
    }

    @Override
    public Collection<InputPort> getInputPorts(Node node) {
        return Collections.unmodifiableCollection(inputPorts.get(node));
    }

    @Override
    public void getInputPortStatus(Node node, int port) {
    }

    @Override
    public Collection<LightPort> getLightPorts(Node node) {
        return Collections.unmodifiableCollection(lightPorts.get(node));
    }

    @Override
    public int getMacroLength(Node node) {
        return 20;
    }

    @Override
    public Macro getMacroContent(Node node, Macro macro) {
        return macro;
    }

    @Override
    public Collection<Macro> getMacros(Node node) {
        return Collections.unmodifiableCollection(macros.get(node));
    }

    @Override
    public Collection<MotorPort> getMotorPorts(Node node) {
        return Collections.unmodifiableCollection(motorPorts.get(node));
    }

    @Override
    public Collection<Node> getNodes() {
        return Collections.unmodifiableCollection(nodes);
    }

    @Override
    public ProtocolVersion getProtocolVersion(Node node) {
        return new ProtocolVersion(0, 4);
    }

    @Override
    public Collection<ServoPort> getServoPorts(Node node) {
        return Collections.unmodifiableCollection(servoPorts.get(node));
    }

    @Override
    public SoftwareVersion getSoftwareVersion(Node node) {
        return new SoftwareVersion('1', '2', '3');
    }

    @Override
    public Collection<SoundPort> getSoundPorts(Node node) {
        return Collections.unmodifiableCollection(soundPorts.get(node));
    }

    @Override
    public int getStorableMacroCount(Node node) {
        return macros.size();
    }

    @Override
    public Collection<SwitchPort> getSwitchPorts(Node node) {
        return Collections.unmodifiableCollection(switchPorts.get(node));
    }

    @Override
    public void identify(Node node, IdentifyState state) {
        LOGGER.info("identify node " + node + ":" + state);
    }

    private org.bidib.wizard.mvc.main.model.Node initNode(byte[] addr, long uniqueId) {
        org.bidib.wizard.mvc.main.model.Node result = new org.bidib.wizard.mvc.main.model.Node();

        result.setNode(new Node(0, addr, uniqueId));
        accessories.put(result.getNode(), new LinkedList<Accessory>());
        analogPorts.put(result.getNode(), new LinkedList<AnalogPort>());
        feedbackPorts.put(result.getNode(), new LinkedList<FeedbackPort>());
        inputPorts.put(result.getNode(), new LinkedList<InputPort>());
        lightPorts.put(result.getNode(), new LinkedList<LightPort>());
        macros.put(result.getNode(), new LinkedList<Macro>());
        motorPorts.put(result.getNode(), new LinkedList<MotorPort>());
        servoPorts.put(result.getNode(), new LinkedList<ServoPort>());
        soundPorts.put(result.getNode(), new LinkedList<SoundPort>());
        switchPorts.put(result.getNode(), new LinkedList<SwitchPort>());
        return result;
    }

    @Override
    public Boolean isAddressMessagesEnabled(Node node) {
        return true;
    }

    @Override
    public Boolean isCommandStation(Node node) {
        return node.getUniqueId() == UUID_ROOT;
    }

    @Override
    public Boolean isDccStartEnabled(Node node) {
        return true;
    }

    @Override
    public Boolean isExternalStartEnabled(Node node) {
        return true;
    }

    @Override
    public Boolean isFeedbackMessagesEnabled(Node node) {
        return true;
    }

    @Override
    public Boolean isKeyMessagesEnabled(Node node) {
        return true;
    }

    @Override
    public boolean isUpdatable(Node node) {
        return node.getUniqueId() == UUID_WOLFGANG;
    }

    @Override
    public int readCv(Node node, int cvNumber) {
        Integer result = cvList.get(cvNumber);

        return result != null ? result : 0;
    }

    @Override
    public void reloadAccessory(Node node, Accessory accessory) {
        LOGGER.info("reload accessory " + accessory + " from node " + node);
    }

    @Override
    public void reloadMacro(Node node, Macro macro) {
        LOGGER.info("reload macro " + macro + " from node " + node);
    }

    @Override
    public void saveAccessory(Node node, Accessory accessory) {
        LOGGER.info("save accessory " + accessory + " to node " + node);
    }

    @Override
    public void saveMacro(Node node, Macro macro) {
        LOGGER.info("save macro " + macro + " to node " + node);
    }

    @Override
    public FirmwareUpdateStat sendFirmwareUpdateOperation(Node node, FirmwareUpdateOperation operation, byte... data) {
        FirmwareUpdateStat result = null;

        LOGGER.info("send firmware update operation " + operation + ", data " + Arrays.toString(data) + " to node "
            + node);
        if (operation == FirmwareUpdateOperation.ENTER) {
            result = new FirmwareUpdateStat(FirmwareUpdateState.READY, 0);
        }
        else if (operation == FirmwareUpdateOperation.SETDEST || operation == FirmwareUpdateOperation.DATA) {
            result = new FirmwareUpdateStat(FirmwareUpdateState.DATA, 1);
        }
        return result;
    }

    @Override
    public void sendTime(Calendar time, int factor) {
        LOGGER.info("send time " + time.getTime() + " with factor " + factor);
    }

    @Override
    public void setAddressMessagesEnabled(Node node, boolean enabled) {
        LOGGER.info("set address messages enabled to " + enabled);
    }

    @Override
    public void setBinaryState(Node node, int address, int state, boolean value) {
        LOGGER.info("set binary state (address=" + address + ",state=" + state + ",value=" + value + ") on node "
            + node);
    }

    @Override
    public void setDccStartEnabled(Node node, boolean enabled) {
        LOGGER.info("set DCC start enabled to " + enabled);
    }

    @Override
    public void setExternalStartEnabled(Node node, boolean enabled) {
        LOGGER.info("set external start enabled to " + enabled);
    }

    @Override
    public void setFeedbackMessagesEnabled(Node node, boolean enabled) {
        LOGGER.info("set feedback messages enabled to " + enabled);
    }

    @Override
    public void setKeyMessagesEnabled(Node node, boolean enabled) {
        LOGGER.info("set key messages enabled to " + enabled);
    }

    @Override
    public void setLightPortParameters(Node node, int port, int pwmMin, int pwmMax, int dimMin, int dimMax) {
        LOGGER.info("values of light node " + node + " port " + port + " changed to [" + pwmMin + "," + pwmMax + ","
            + dimMin + "," + dimMax + "]");
        lightPorts.get(node).get(port).setDimMin(dimMin);
        lightPorts.get(node).get(port).setDimMax(dimMax);
        lightPorts.get(node).get(port).setPwmMin(pwmMin);
        lightPorts.get(node).get(port).setPwmMax(pwmMax);
    }

    @Override
    public void setServoPortParameters(Node node, int port, int trimDown, int trimUp, int speed) {
        LOGGER.info("values of servo node " + node + " port " + port + " changed to [" + trimDown + "," + trimUp + ","
            + speed + "]");
        servoPorts.get(node).get(port).setSpeed(speed);
        servoPorts.get(node).get(port).setTrimDown(trimDown);
        servoPorts.get(node).get(port).setTrimUp(trimUp);
    }

    @Override
    public void setSpeed(
        Node node, int address, SpeedSteps speedSteps, Integer speed, Direction direction, BitSet activeFunctions,
        BitSet functions) {
        LOGGER.info("set speed values (address=" + address + ",speedSteps=" + speedSteps + ",speed=" + speed
            + ",direction=" + direction + ",activeFunctions=" + activeFunctions + ",functions=" + functions
            + ") on node " + node);
    }

    @Override
    public void startAccessory(Node node, Accessory accessory, int aspect) {
        LOGGER.info("start aspect " + aspect + " of accessory " + accessory + " on node " + node);
    }

    @Override
    public void startMacro(Node node, Macro macro) {
        LOGGER.info("start macro " + macro + " on node " + node);
    }

    @Override
    public void stopMacro(Node node, Macro macro) {
        LOGGER.info("stop macro " + macro + " on node " + node);
    }

    @Override
    public void sysDisable(Node node) {
        LOGGER.info("disable " + node);
    }

    @Override
    public void sysEnable(Node node) {
        LOGGER.info("enable " + node);
    }

    @Override
    public void transferMacro(Node node, Macro macro) {
        LOGGER.info("transfer macro " + macro + " to node " + node);
        LOGGER.info("speed: " + macro.getSpeed());
        LOGGER.info("cycles: " + macro.getCycles());
        LOGGER.info("startCondition: " + macro.getStartConditions());
        for (Function function : macro.getFunctions()) {
            if (function != null) {
                LOGGER.info(function.getDebugString());
            }
        }
    }

    @Override
    public void writeCv(Node node, int cvNumber, int cvValue) {
        cvList.put(cvNumber, cvValue);
    }
}
