package org.bidib.wizard.mvc.cvprogrammer.view.panel;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.DecimalFormat;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.NumberFormatter;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.cvprogrammer.model.CvProgrammerModel;
import org.bidib.wizard.mvc.cvprogrammer.model.listener.ConfigVariableListener;
import org.bidib.wizard.mvc.cvprogrammer.view.panel.listener.CvWriteListener;

public class CvWritePanel extends JPanel {
    private static final long serialVersionUID = 7548794684666991097L;

    private final Collection<CvWriteListener> listeners = new LinkedList<CvWriteListener>();

    private final JLabel cvValueLabel = new JLabel(Resources.getString(getClass(), "value") + ":");

    private final NumberFormatter formatter = new NumberFormatter(
        new DecimalFormat("#")) {
        private static final long serialVersionUID = 7803948875353855337L;

        {
            setAllowsInvalid(false);
            setMaximum(255);
            setMinimum(0);
        }
    };

    private final JFormattedTextField cvValue = new JFormattedTextField();

    private final JCheckBox[] bitValues = new JCheckBox[8];

    private final JButton writeButton = new JButton(Resources.getString(getClass(), "write"));

    private final DocumentListener documentListener = new myDocumentListener();

    public CvWritePanel(final CvProgrammerModel model) {
        setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Resources.getString(getClass(),
            "title")
            + ":"));
        setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.NONE;
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        add(cvValueLabel, c);

        c.gridx++;
        cvValue.setColumns(3);
        cvValue.setFormatterFactory(new DefaultFormatterFactory(formatter, formatter, formatter));
        cvValue.setValue(0);
        add(cvValue, c);

        for (int bit = 7; bit >= 0; bit--) {
            bitValues[bit] = new JCheckBox(String.valueOf(bit), false);
            bitValues[bit].addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    // set integer value from bit values
                    final StringBuffer bitValue = new StringBuffer();

                    for (int bit = bitValues.length - 1; bit >= 0; bit--) {
                        bitValue.append(bitValues[bit].isSelected() ? '1' : '0');
                    }
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            cvValue.getDocument().removeDocumentListener(documentListener);
                            model.setValue(Integer.valueOf(bitValue.toString(), 2));
                            cvValue.getDocument().addDocumentListener(documentListener);
                        }
                    });
                }
            });
            c.gridx += 1;
            add(bitValues[bit], c);
        }
        cvValue.getDocument().addDocumentListener(documentListener);

        writeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                model.setValue(Integer.parseInt(cvValue.getText()));
                fireWrite(model.getValue());
            }
        });

        c.anchor = GridBagConstraints.FIRST_LINE_END;
        c.gridx++;
        c.weightx = 1;
        add(writeButton, c);
        model.addConfigVariableListener(new ConfigVariableListener() {
            @Override
            public void numberChanged(int number) {
            }

            @Override
            public void valueChanged(int value) {
                cvValue.setValue(value);
            }
        });
    }

    public void addCvWriteListener(CvWriteListener l) {
        listeners.add(l);
    }

    private void fireWrite(int cvValue) {
        for (CvWriteListener l : listeners) {
            l.write(cvValue);
        }
    }

    public int getButtonWidth() {
        return writeButton.getWidth();
    }

    public int getLabelWidth() {
        return cvValueLabel.getWidth();
    }

    public void setButtonWidth(int width) {
        writeButton.setPreferredSize(new Dimension(width, writeButton.getSize().height));
    }

    public void setLabelWidth(int width) {
        cvValueLabel.setPreferredSize(new Dimension(width, cvValueLabel.getSize().height));
    }

    private void valueChanged(String value) {
        try {
            String bitValue = Integer.toBinaryString(Integer.parseInt(value));

            for (int index = 0; index < bitValues.length; index++) {
                if (index < bitValue.length()) {
                    bitValues[index].setSelected(bitValue.charAt(bitValue.length() - index - 1) == '1');
                }
                else {
                    bitValues[index].setSelected(false);
                }
            }
        }
        catch (NumberFormatException ex) {
            for (int index = 0; index < bitValues.length; index++) {
                bitValues[index].setSelected(false);
            }
        }
    }

    private class myDocumentListener implements DocumentListener {
        public void changedUpdate(DocumentEvent e) {
            valueChanged(cvValue.getText());
        }

        public void insertUpdate(DocumentEvent e) {
            valueChanged(cvValue.getText());
        }

        public void removeUpdate(DocumentEvent e) {
            valueChanged(cvValue.getText());
        }
    }
}
