package org.bidib.wizard.mvc.firmware.controller;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.JFrame;

import org.bidib.jbidibc.FirmwareUpdateStat;
import org.bidib.jbidibc.enumeration.FirmwareUpdateOperation;
import org.bidib.jbidibc.enumeration.FirmwareUpdateState;
import org.bidib.jbidibc.utils.ByteUtils;
import org.bidib.wizard.comm.Communication;
import org.bidib.wizard.comm.CommunicationFactory;
import org.bidib.wizard.mvc.firmware.controller.listener.FirmwareControllerListener;
import org.bidib.wizard.mvc.firmware.model.FirmwareModel;
import org.bidib.wizard.mvc.firmware.model.MemoryType;
import org.bidib.wizard.mvc.firmware.view.FirmwareView;
import org.bidib.wizard.mvc.firmware.view.listener.FirmwareViewListener;
import org.bidib.wizard.mvc.main.model.Node;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class FirmwareController {
    private static final Logger LOG = LoggerFactory.getLogger(FirmwareController.class.getName());

    private final Collection<FirmwareControllerListener> listeners = new LinkedList<FirmwareControllerListener>();

    private final Node node;

    private final JFrame parent;

    private final int x;

    private final int y;

    private FirmwareModel model = new FirmwareModel();

    private File file = null;

    private MemoryType type = null;

    public FirmwareController(JFrame parent, Node node, int x, int y) {
        this.parent = parent;
        this.node = node;
        this.x = x;
        this.y = y;
    }

    public void addFirmwareControllerListener(FirmwareControllerListener l) {
        listeners.add(l);
    }

    private void fireClose() {
        for (FirmwareControllerListener l : listeners) {
            l.close();
        }
    }

    public void start() {
        final Communication communication = CommunicationFactory.getInstance();

        // get current values
        model.setUniqueId(node.getNode().getUniqueIdAsString());
        model.setVersion(communication.getSoftwareVersion(node.getNode()).toString());

        final FirmwareView view = new FirmwareView(parent, model, x, y);

        view.addFirmwareViewListener(new FirmwareViewListener() {
            @Override
            public void chooseFirmware(File file, MemoryType type) {
                setFile(file);
                setMemoryType(type);
                model.setProgressValue(0);
            }

            @Override
            public void close() {
                try {
                    sendCommand(CommunicationFactory.getInstance(), FirmwareUpdateOperation.EXIT);
                }
                catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                fireClose();
            }

            @Override
            public void update() {
                if (file != null) {
                    new Thread() {
                        @Override
                        public void run() {
                            BufferedReader input = null;
                            final int fileSize = (int) file.length();
                            int currentSize = 0;

                            try {
                                if (sendCommand(communication, FirmwareUpdateOperation.ENTER, ByteUtils
                                    .convertLongToUniqueId(node.getNode().getUniqueId()))) {
                                    if (sendCommand(communication, FirmwareUpdateOperation.SETDEST, type.getType())) {
                                        input = new BufferedReader(new FileReader(file));

                                        String line = null;

                                        while ((line = input.readLine()) != null) {
                                            currentSize += line.length();
                                            model.setProgressValue(currentSize * 100 / fileSize);
                                            if (!sendCommand(communication, FirmwareUpdateOperation.DATA, line
                                                .trim().getBytes())) {
                                                LOG.warn("unexpected answer from node");
                                                break;
                                            }
                                        }
                                        sendCommand(communication, FirmwareUpdateOperation.DONE);
                                        model.setProgressValue(100);
                                        model.setInProgress(false);
                                    }
                                }
                            }
                            catch (Exception e) {
                                throw new RuntimeException(e);
                            }
                            finally {
                                if (input != null) {
                                    try {
                                        input.close();
                                    }
                                    catch (IOException e) {
                                        throw new RuntimeException(e);
                                    }
                                }
                            }
                        }
                    }.start();
                }
                else {
                    model.setInProgress(false);
                }
            }
        });
    }

    private boolean sendCommand(Communication communication, FirmwareUpdateOperation operation, byte... data)
        throws InterruptedException {
        boolean result = false;
        FirmwareUpdateStat updateStat = communication.sendFirmwareUpdateOperation(node.getNode(), operation, data);

        if (updateStat != null) {
            Thread.sleep(updateStat.getTimeout() * 10);

            FirmwareUpdateState state = updateStat.getState();

            if ((operation == FirmwareUpdateOperation.ENTER && state == FirmwareUpdateState.READY)
                || (operation == FirmwareUpdateOperation.SETDEST && state == FirmwareUpdateState.DATA)
                || (operation == FirmwareUpdateOperation.DATA && state == FirmwareUpdateState.DATA)) {
                result = true;
            }
        }
        return result;
    }

    private void setFile(File file) {
        this.file = file;
    }

    private void setMemoryType(MemoryType type) {
        this.type = type;
    }
}
