package org.bidib.wizard.mvc.firmware.view.panel;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

import org.bidib.wizard.dialog.FileDialog;
import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.firmware.model.MemoryType;
import org.bidib.wizard.mvc.firmware.view.panel.listener.FirmwareChooseListener;

public class FirmwareChoosePanel extends JPanel {
    private static final long serialVersionUID = 4184610210025666280L;

    private final Collection<FirmwareChooseListener> listeners = new LinkedList<FirmwareChooseListener>();

    private final JLabel fileLabel = new JLabel(Resources.getString(getClass(), "file") + ":");

    private final JButton fileButton = new JButton(Resources.getString(getClass(), "chooseFile"));

    private final JTextField version = new JTextField();

    private final JTextField memoryType = new JTextField();

    public FirmwareChoosePanel() {
        setBorder(BorderFactory.createEtchedBorder());
        setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        add(fileLabel, c);

        final JTextField fileName = new JTextField(30);

        fileName.setEditable(false);
        c.gridx++;
        c.weightx = 1;
        add(fileName, c);

        fileButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                FileFilter ff = new FileFilter() {
                    private static final String SUFFIX = "hex";

                    @Override
                    public boolean accept(File file) {
                        boolean result = false;

                        if (file != null) {
                            if (file.isDirectory()) {
                                result = true;
                            }
                            else if (file.toString() != null) {
                                String[] parts = file.toString().split("\\.");

                                if (parts != null && parts.length >= 2
                                    && parts[parts.length - 1].toLowerCase().equals(SUFFIX)
                                    && parts[parts.length - 2].matches("\\d{3}")) {
                                    try {
                                        MemoryType.valueOf(Integer.parseInt(parts[parts.length - 2]));
                                        result = true;
                                    }
                                    catch (Exception e) {
                                    }
                                }
                            }
                        }
                        return result;
                    }

                    @Override
                    public String getDescription() {
                        return Resources.getString(FirmwareChoosePanel.class, "filter") + " (*." + SUFFIX + ")";
                    }
                };

                new FileDialog(
                    FileDialog.OPEN, null, ff) {
                    @Override
                    public void approve(String selectedFile) {
                        File file = new File(selectedFile);

                        selectedFile = file.getName();

                        String[] parts = selectedFile.split("\\.");
                        int endIndex = selectedFile.indexOf(parts[parts.length - 2]) - 1;

                        if (endIndex > 0) {
                            int beginIndex = selectedFile.substring(0, endIndex).lastIndexOf('_') + 1;

                            version.setText(selectedFile.substring(beginIndex, endIndex));
                        }

                        MemoryType type = MemoryType.valueOf(Integer.parseInt(parts[parts.length - 2]));

                        fileName.setText(file.toString());
                        memoryType.setText(type.name());

                        fireChooseFirmware(file, type);
                    }
                };
            }
        });
        c.gridx++;
        c.weightx = 0;
        add(fileButton, c);

        c.gridx = 0;
        c.gridy++;
        c.weighty = 1;
        add(new JLabel(Resources.getString(getClass(), "version") + ":"), c);

        version.setEditable(false);
        c.gridx++;
        add(version, c);

        c.gridx = 0;
        c.gridy++;
        c.weighty = 1;
        add(new JLabel(Resources.getString(getClass(), "memoryType") + ":"), c);

        memoryType.setEditable(false);
        c.gridx++;
        add(memoryType, c);
    }

    public void addFirmwareChooseListener(FirmwareChooseListener l) {
        listeners.add(l);
    }

    private void fireChooseFirmware(File file, MemoryType type) {
        for (FirmwareChooseListener l : listeners) {
            l.chooseFirmware(file, type);
        }
    }

    public int getButtonWidth() {
        return fileButton.getWidth();
    }

    public int getLabelWidth() {
        return fileLabel.getWidth();
    }

    public void setButtonWidth(int width) {
        fileButton.setPreferredSize(new Dimension(width, fileButton.getSize().height));
    }
}
