package org.bidib.wizard.mvc.firmware.view.panel;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.firmware.model.FirmwareModel;
import org.bidib.wizard.mvc.firmware.model.listener.FirmwareModelListener;
import org.bidib.wizard.mvc.firmware.view.panel.listener.FirmwareProcessListener;

public class FirmwareProcessPanel extends JPanel {
    private static final long serialVersionUID = 4184610210025666280L;

    private final Collection<FirmwareProcessListener> listeners = new LinkedList<FirmwareProcessListener>();

    private final JLabel progressLabel = new JLabel(Resources.getString(getClass(), "progress") + ":");

    private final JProgressBar progressBar = new JProgressBar(0, 100);

    private final JButton startButton = new JButton(Resources.getString(getClass(), "startUpdate"));

    public FirmwareProcessPanel(FirmwareModel model) {
        setBorder(BorderFactory.createEtchedBorder());
        setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weighty = 1;
        add(progressLabel, c);

        progressBar.setStringPainted(true);
        c.gridx++;
        c.weightx = 1;
        add(progressBar, c);

        startButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireUpdate();
            }
        });
        c.gridx++;
        c.weightx = 0;
        add(startButton, c);

        model.addFirmwareModelListener(new FirmwareModelListener() {
            @Override
            public void inProgressChanged(boolean inProgress) {
                startButton.setEnabled(!inProgress);
            }

            @Override
            public void progressValueChanged(final int progressValue) {
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        progressBar.setValue(progressValue);
                    }
                });
            }
        });
    }

    public void addFirmwareProcessListener(FirmwareProcessListener l) {
        listeners.add(l);
    }

    private void fireUpdate() {
        for (FirmwareProcessListener l : listeners) {
            l.update();
        }
    }

    public int getButtonWidth() {
        return startButton.getWidth();
    }

    public void setButtonWidth(int width) {
        startButton.setPreferredSize(new Dimension(width, startButton.getSize().height));
    }

    public void setLabelWidth(int width) {
        progressLabel.setPreferredSize(new Dimension(width, progressLabel.getSize().height));
    }
}
