package org.bidib.wizard.mvc.loco.controller;

import java.util.BitSet;
import java.util.Collection;

import javax.swing.JFrame;

import org.bidib.jbidibc.AddressData;
import org.bidib.jbidibc.MessageListener;
import org.bidib.jbidibc.MessageReceiver;
import org.bidib.jbidibc.enumeration.BoosterState;
import org.bidib.wizard.comm.Communication;
import org.bidib.wizard.comm.CommunicationFactory;
import org.bidib.wizard.comm.Direction;
import org.bidib.wizard.comm.SpeedSteps;
import org.bidib.wizard.mvc.loco.model.LocoModel;
import org.bidib.wizard.mvc.loco.model.listener.LocoModelListener;
import org.bidib.wizard.mvc.loco.view.LocoView;
import org.bidib.wizard.mvc.main.model.Node;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class LocoController {
    private static final Logger LOGGER = LoggerFactory.getLogger(LocoController.class);

    private final Node node;

    private final JFrame parent;

    private final int x;

    private final int y;

    private final LocoModel model = new LocoModel();

    public LocoController(Node node, JFrame parent, int x, int y) {
        this.node = node;
        this.parent = parent;
        this.x = x;
        this.y = y;
    }

    public void start() {
        final Communication communication = CommunicationFactory.getInstance();
        MessageReceiver.addMessageListener(new MessageListener() {
            @Override
            public void address(byte[] address, int detectorNumber, Collection<AddressData> addressData) {
            }

            @Override
            public void boosterCurrent(byte[] address, int current) {
            }

            @Override
            public void boosterState(byte[] address, BoosterState state) {
            }

            @Override
            public void boosterTemperature(byte[] address, int temperature) {
            }

            @Override
            public void boosterVoltage(byte[] address, int voltage) {
            }

            @Override
            public void confidence(byte[] address, int valid, int freeze, int signal) {
            }

            @Override
            public void free(byte[] address, int detectorNumber) {
            }

            @Override
            public void key(byte[] address, int keyNumber, int keyState) {
            }

            @Override
            public void nodeLost(org.bidib.jbidibc.Node node) {
            }

            @Override
            public void nodeNew(org.bidib.jbidibc.Node node) {
            }

            @Override
            public void occupied(byte[] address, int detectorNumber) {
            }

            @Override
            public void speed(byte[] address, AddressData addressData, int speed) {
                if (addressData.getAddress() == model.getAddress()) {
                    model.setCurrentSpeed(speed);
                }
            }
        });

        model.addLocoModelListener(new LocoModelListener() {
            @Override
            public void addressChanged(int address) {
            }

            @Override
            public void currentSpeedChanged(int currentSpeed) {
            }

            @Override
            public void directionChanged(Direction direction) {
            }

            @Override
            public void functionChanged(int index, boolean value) {
                int functionGroupIndex = (index + 3) / 8;
                BitSet activeFunctions = new BitSet(functionGroupIndex + 1);
                LOGGER.debug("functions have changed, functionGroupIndex: {}", functionGroupIndex);
                activeFunctions.set(functionGroupIndex, true);
                communication.setSpeed(node.getNode(), model.getAddress(), model.getSpeedSteps(), null, model
                    .getDirection(), activeFunctions, model.getFunctions());
            }

            @Override
            public void speedChanged(int speed) {
                communication.setSpeed(node.getNode(), model.getAddress(), model.getSpeedSteps(), speed, model
                    .getDirection(), null, null);
            }

            @Override
            public void speedStepsChanged(SpeedSteps speedSteps) {
            }
        });
        new LocoView(parent, model, x, y);
    }
}
