package org.bidib.wizard.mvc.loco.view;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.bidib.wizard.comm.Direction;
import org.bidib.wizard.comm.SpeedSteps;
import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.loco.model.LocoModel;
import org.bidib.wizard.mvc.loco.model.listener.LocoModelListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class LocoView extends JDialog {
    private static final Logger LOGGER = LoggerFactory.getLogger(LocoView.class);

    private static final long serialVersionUID = -1788605917784225918L;

    public LocoView(JFrame parent, final LocoModel model, int x, int y) {
        super(parent);
        setLayout(new GridBagLayout());
        setLocation(x, y);
        setTitle(Resources.getString(getClass(), "title"));

        JPanel infoPanel = new JPanel(new GridBagLayout());

        infoPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Resources.getString(
            getClass(), "loco")
            + ":"));

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 0;

        infoPanel.add(new JLabel(Resources.getString(getClass(), "address") + ":"), c);

        final JTextField address = new JFormattedTextField(NumberFormat.getNumberInstance());

        address.getDocument().addDocumentListener(new DocumentListener() {
            @Override
            public void changedUpdate(DocumentEvent e) {
                valueChanged();
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                valueChanged();
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                valueChanged();
            }

            private void valueChanged() {
                if (!address.getText().isEmpty()) {
                    try {
                        model.setAddress(Integer.parseInt(address.getText()));
                    }
                    catch (NumberFormatException e) {
                    }
                }
            }
        });
        address.setText("3");

        c.gridx++;
        c.weightx = 1;
        infoPanel.add(address, c);

        c.gridx++;
        c.weightx = 0;
        infoPanel.add(new JLabel(Resources.getString(getClass(), "speedSteps") + ":"), c);

        JComboBox speedSteps = new JComboBox(SpeedSteps.values());

        speedSteps.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                model.setSpeedSteps((SpeedSteps) ((JComboBox) e.getSource()).getSelectedItem());
            }
        });
        speedSteps.setSelectedIndex(model.getSpeedSteps().ordinal());
        c.gridx++;
        infoPanel.add(speedSteps, c);

        c.gridwidth = 3;
        c.gridx = 0;
        c.weightx = 1;
        add(infoPanel, c);

        JPanel directionPanel = new JPanel(new GridBagLayout());

        directionPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Resources
            .getString(getClass(), "speed")
            + ":"));

        final JSlider speedSlider = new JSlider(JSlider.HORIZONTAL);

        speedSlider.setMaximum(model.getSpeedSteps().getSteps());
        speedSlider.setMinimum(-model.getSpeedSteps().getSteps());
        speedSlider.setMajorTickSpacing(speedSlider.getMaximum());
        speedSlider.setPaintTicks(true);
        speedSlider.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                if (!speedSlider.getValueIsAdjusting()) {
                    int value = (int) speedSlider.getValue();

                    if (value > 0) {
                        model.setDirection(Direction.FORWARD);
                        value++;
                    }
                    else if (value < 0) {
                        model.setDirection(Direction.BACKWARD);
                        value--;
                    }
                    value = Math.abs(value);
                    LOGGER.debug("Set the speed value: {}", value);
                    model.setSpeed(value);
                }
            }
        });
        speedSlider.setValue(0);

        c.gridx = 0;
        c.weightx = 1;
        directionPanel.add(speedSlider, c);

        c.fill = GridBagConstraints.NONE;
        c.gridwidth = 1;
        c.gridy++;
        directionPanel.add(new JLabel(Resources.getString(getClass(), "backwards")), c);

        final JLabel reportedSpeed = new JLabel(String.valueOf(model.getCurrentSpeed()));

        c.anchor = GridBagConstraints.CENTER;
        c.gridx++;
        directionPanel.add(reportedSpeed, c);

        c.anchor = GridBagConstraints.FIRST_LINE_END;
        c.gridx++;
        directionPanel.add(new JLabel(Resources.getString(getClass(), "forwards")), c);

        c.anchor = GridBagConstraints.CENTER;
        c.fill = GridBagConstraints.NONE;
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx = 0;
        c.gridy++;
        directionPanel.add(new JLabel(Resources.getString(getClass(), "reportedSpeed")), c);

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridwidth = 3;
        c.gridx = 0;
        c.gridy++;
        add(directionPanel, c);

        final List<JToggleButton> buttons = new LinkedList<JToggleButton>();
        JPanel f0ButtonPanel = new JPanel(new GridBagLayout());

        f0ButtonPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Resources
            .getString(getClass(), "light")
            + ":"));

        final JToggleButton f0Button = new JToggleButton("*");

        f0Button.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                model.setFunction(0, f0Button.isSelected());
            }
        });
        buttons.add(f0Button);

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridwidth = 1;
        c.gridy++;
        c.weighty = 1;
        f0ButtonPanel.add(f0Button, c);

        c.weightx = 0.1;
        add(f0ButtonPanel, c);

        JPanel stopPanel = new JPanel(new GridBagLayout());

        stopPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Resources.getString(
            getClass(), "stopLoco")
            + ":"));

        JButton stopButton = new JButton(Resources.getString(getClass(), "stop"));

        stopButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                model.setSpeed(0);
            }
        });

        c.weightx = 0.5;
        stopPanel.add(stopButton, c);

        JButton emergencyStopButton = new JButton(Resources.getString(getClass(), "emergencyStop"));

        emergencyStopButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                model.setSpeed(1);
            }
        });

        c.gridx++;
        stopPanel.add(emergencyStopButton, c);

        c.anchor = GridBagConstraints.FIRST_LINE_END;
        c.gridx = 2;
        c.weightx = 0.9;
        add(stopPanel, c);

        JPanel functionPanel = new JPanel(new GridBagLayout());

        functionPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Resources
            .getString(getClass(), "additionalFunctions")
            + ":"));

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridwidth = 1;
        c.weightx = 1;
        for (int row = 0; row < 4; row++) {
            c.gridx = 0;
            c.gridy++;
            for (int column = 0; column < 7; column++) {
                final int functionIndex = row * 7 + (column + 1);
                final JToggleButton functionButton = new JToggleButton("F" + functionIndex);

                functionButton.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        model.setFunction(functionIndex, functionButton.isSelected());
                    }
                });
                buttons.add(functionButton);

                functionPanel.add(functionButton, c);
                c.gridx++;
            }
        }

        c.gridwidth = 3;
        c.gridx = 0;
        c.gridy++;
        add(functionPanel, c);

        model.addLocoModelListener(new LocoModelListener() {
            @Override
            public void addressChanged(int address) {
                reportedSpeed.setText("");
            }

            @Override
            public void currentSpeedChanged(int currentSpeed) {
                reportedSpeed.setText(String.valueOf(currentSpeed));
            }

            @Override
            public void directionChanged(Direction direction) {
            }

            @Override
            public void functionChanged(int index, boolean value) {
                buttons.get(index).setSelected(value);
            }

            @Override
            public void speedChanged(int speed) {
                if (speed == 0 || speed == 1) {
                    speedSlider.setValue(0);
                }
            }

            @Override
            public void speedStepsChanged(SpeedSteps speedSteps) {
                speedSlider.setMaximum(speedSteps.getSteps());
                speedSlider.setMinimum(-speedSteps.getSteps());
                speedSlider.setMajorTickSpacing(speedSlider.getMaximum());
            }
        });

        pack();
        setMinimumSize(getSize());
        setVisible(true);
    }
}
