package org.bidib.wizard.mvc.main.model;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.listener.AccessoryListener;

public class Accessory implements LabelAware {
    private final Collection<AccessoryListener> listeners = new LinkedList<AccessoryListener>();

    private int id = 0;

    private String label = null;

    private Vector<Integer> macros = new Vector<Integer>();

    private int macroSize = 0;

    public void addAccessoryListener(AccessoryListener l) {
        listeners.add(l);
    }

    public void removeAccessoryListener(AccessoryListener l) {
        listeners.remove(l);
    }

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public void addMacro(Macro macro) {
        addMacro(macros.size(), macro);
    }

    private boolean addMacro(int index, Macro macro) {
        boolean result = false;

        if (macro != null && index >= 0 && macros.size() < macroSize) {
            macros.add(index, macro.getId());
            fireMacrosChanged();
            result = true;
        }
        return result;
    }

    public void addMacroAfter(int index, Macro macro) {
        addMacro(index + 1, macro);
    }

    public void addMacroBefore(int index, Macro macro) {
        addMacro(index, macro);
    }

    public void clearMacros() {
        macros.clear();
        fireMacrosChanged();
    }

    public int getMacroCount() {
        return macros.size();
    }

    public List<Integer> getMacros() {
        return Collections.unmodifiableList(macros);
    }

    public void removeMacro(int index) {
        macros.remove(index);
        fireMacrosChanged();
    }

    public void setMacro(int index, int macro) {
        macros.set(index, macro);
        fireMacrosChanged();
    }

    public void setMacros(List<Integer> macros) {
        this.macros = new Vector<Integer>(macros);
    }

    public int getMacroSize() {
        return macroSize;
    }

    public void setMacroSize(int macroSize) {
        this.macroSize = macroSize;
        if (this.macros.size() > macroSize) {
            this.macros.setSize(macroSize);
        }
    }

    private void fireMacrosChanged() {
        for (AccessoryListener l : listeners) {
            l.macrosChanged();
        }
    }

    public String getDebugString() {
        return getClass().getSimpleName() + "[id=" + id + ",label=" + label + ",macros=" + macros + ",macroSize="
            + macroSize + "]";
    }

    public static Accessory load(String fileName) {
        Accessory result = null;
        XMLDecoder d = null;

        try {
            try {
                d =
                    new XMLDecoder(XmlLoader.changePackage(new InputStreamReader(new GZIPInputStream(
                        new FileInputStream(fileName)))));
                result = (Accessory) d.readObject();
            }
            catch (IOException e) {
                d = new XMLDecoder(XmlLoader.changePackage(new InputStreamReader(new FileInputStream(fileName))));
                result = (Accessory) d.readObject();
            }
        }
        catch (FileNotFoundException e) {
        }
        finally {
            if (d != null) {
                d.close();
            }
        }
        return result;
    }

    public void save(String fileName) throws IOException {
        XMLEncoder e = new XMLEncoder(new GZIPOutputStream(new BufferedOutputStream(new FileOutputStream(fileName))));

        e.writeObject(this);
        e.close();
    }

    public String toString() {
        String result = "";

        if (label != null && label.length() > 0) {
            result = label;
        }
        else {
            result = Resources.getString(getClass(), "label") + "_" + id;
        }
        return result;
    }
}
