package org.bidib.wizard.mvc.main.model;

import javax.swing.table.DefaultTableModel;

import org.bidib.wizard.comm.BidibStatus;
import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.function.Delayable;
import org.bidib.wizard.mvc.main.model.function.FlagFunction;
import org.bidib.wizard.mvc.main.model.function.Function;
import org.bidib.wizard.mvc.main.model.function.InputFunction;
import org.bidib.wizard.mvc.main.model.function.MacroFunction;
import org.bidib.wizard.mvc.main.model.function.PortAction;
import org.bidib.wizard.mvc.main.model.function.RandomDelayFunction;
import org.bidib.wizard.mvc.main.model.function.ServoPortAction;
import org.bidib.wizard.mvc.main.model.listener.MacroListener;

public class MacroTableModel extends DefaultTableModel implements MacroListener {
    private static final long serialVersionUID = -4967713740905049760L;

    private final static String[] COLUMN_NAMES = new String[6];

    private Macro macro = null;

    private final MainModel model;

    public MacroTableModel(MainModel model) {
        this.model = model;
        setColumnIdentifiers(COLUMN_NAMES);
    }

    private void addRow(Function function, int functionIndex) {
        Object[] rowData = new Object[COLUMN_NAMES.length];

        for (int column = 0; column < COLUMN_NAMES.length; column++) {
            rowData[column] = getRowData(column, function, functionIndex);
        }
        addRow(rowData);
    }

    @Override
    public Class<?> getColumnClass(int column) {
        switch (column) {
            case 1:
                return Integer.class;
            case 2:
                return Object.class;
            case 3:
                return Object.class;
            case 4:
                return Object.class;
            case 5:
                return Integer.class;
            default:
                return String.class;
        }
    }

    private Object getRowData(int column, Function function, int functionIndex) {
        Object result = null;

        switch (column) {
            case 0:
                result = (functionIndex + 1) + ". " + Resources.getString(getClass(), "step");
                break;
            case 1:
                if (function != null) {
                    if (function instanceof Delayable) {
                        result = ((Delayable) function).getDelay();
                    }
                    else if (function instanceof RandomDelayFunction) {
                        result = ((RandomDelayFunction) function).getMaximumValue();
                    }
                }
                break;
            case 2:
                result = function;
                break;
            case 3:
                if (function != null) {
                    result = function.getAction();
                }
                break;
            case 4:
                if (function != null) {
                    if (function instanceof InputFunction) {
                        result = ((InputFunction) function).getInput();
                    }
                    else if (function instanceof MacroFunction) {
                        result = model.getMacros().get(((MacroFunction) function).getMacroId());
                    }
                    else if (function instanceof FlagFunction) {
                        result = ((FlagFunction) function).getFlag();
                    }
                    else if (function instanceof PortAction) {
                        result = ((PortAction) function).getPort();
                    }
                }
                break;
            case 5:
                if (function != null && function instanceof ServoPortAction) {
                    result = ((ServoPortAction) function).getValue();
                }
                break;
        }
        return result;
    }

    @Override
    public Object getValueAt(int row, int column) {
        Object result = super.getValueAt(row, column);

        if (result != null && column == 5) {
            Function function = model.getMacro().getFunction(row);

            if (function instanceof ServoPortAction) {
                ServoPort port = ((ServoPortAction) function).getPort();

                if (port != null) {
                    result = port.getRelativeValue(Integer.parseInt(result.toString()));
                }
            }
        }
        return result;
    }

    private void insertRow(int row, Function function, int functionIndex) {
        Object[] rowData = new Object[COLUMN_NAMES.length];

        for (int column = 0; column < COLUMN_NAMES.length; column++) {
            rowData[column] = getRowData(column, function, functionIndex);
        }
        insertRow(row, rowData);
    }

    @Override
    public boolean isCellEditable(int row, int column) {
        return column > 0;
    }

    private void renumberSteps() {
        for (int row = 0; row < getRowCount(); row++) {
            Object rowData = getRowData(0, (Function) getValueAt(row, 2), row);

            rowData = (row + 1) + ". " + Resources.getString(getClass(), "step");
            setValueAt(rowData, row, 0);

        }
    }

    public void setMacro(Macro macro) {
        setRowCount(0);
        if (this.macro != null) {
            this.macro.removeMacroListener(this);
        }
        this.macro = macro;
        if (macro != null) {
            for (int index = 0; index < model.getMacro().getFunctionCount(); index++) {
                addRow(model.getMacro().getFunction(index), index);
            }
            macro.addMacroListener(this);
        }
    }

    @Override
    public void setValueAt(Object value, int row, int column) {
        switch (column) {
            case 1:
                Function function = model.getMacro().getFunction(row);

                if (function instanceof Delayable) {
                    ((Delayable) function).setDelay(Integer.parseInt(value.toString()));
                    super.setValueAt(value, row, column);
                }
                else if (function instanceof RandomDelayFunction) {
                    ((RandomDelayFunction) function).setMaximumValue(Integer.parseInt(value.toString()));
                    super.setValueAt(value, row, column);
                }
                break;
            case 2:
                if (value != null
                    && (model.getMacro().getFunction(row) == null || value.getClass() != model.getMacro().getFunction(
                        row).getClass())) {
                    try {
                        function = (Function) ((Function) value).clone();
                        model.getMacro().replaceFunction(row, function);
                        for (int col = COLUMN_NAMES.length - 1; col > 0; col--) {
                            super.setValueAt(getRowData(col, function, row), row, col);
                        }
                    }
                    catch (Exception e) {
                        throw new RuntimeException(e);
                    }
                }
                break;
            case 3:
                function = model.getMacro().getFunction(row);
                function.setAction((BidibStatus) value);
                super.setValueAt(value, row, column);
                break;
            case 4:
                function = model.getMacro().getFunction(row);
                if (function instanceof InputFunction) {
                    ((InputFunction) function).setInput((InputPort) value);
                    super.setValueAt(value, row, column);
                }
                else if (function instanceof MacroFunction) {
                    ((MacroFunction) function).setMacroId(model.getMacros().indexOf(value));
                    super.setValueAt(value, row, column);
                }
                else if (function instanceof FlagFunction) {
                    ((FlagFunction) function).setFlag((Flag) value);
                    super.setValueAt(value, row, column);
                }
                else if (function instanceof PortAction) {
                    ((PortAction) function).setPort((Port) value);
                    super.setValueAt(value, row, column);
                }
                break;
            case 5:
                function = model.getMacro().getFunction(row);
                if (function instanceof ServoPortAction) {
                    int intValue =
                        ((ServoPortAction) function).getPort().getAbsoluteValue(Integer.parseInt(value.toString()));

                    ((ServoPortAction) function).setValue(intValue);
                    super.setValueAt(intValue, row, column);
                }
                break;
            default:
                super.setValueAt(value, row, column);
        }
    }

    @Override
    public void functionsAdded(int row, Function[] functions) {
        if (functions != null) {
            for (int index = 0; index < functions.length; index++) {
                insertRow(row + index, functions[index], row);
            }
        }
        else {
            insertRow(row, (Function) null, row);
        }
        renumberSteps();
    }

    @Override
    public void functionRemoved(int row) {
        removeRow(row);
        renumberSteps();
    }

    @Override
    public void functionsRemoved() {
        setRowCount(0);
    }

    @Override
    public void labelChanged(String label) {
    }
}
