package org.bidib.wizard.mvc.main.model;

import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

public class ServoPort extends Port {
    static {
        try {
            PropertyDescriptor[] descriptor = Introspector.getBeanInfo(ServoPort.class).getPropertyDescriptors();

            for (int i = 0; i < descriptor.length; i++) {
                PropertyDescriptor propertyDescriptor = descriptor[i];
                if (propertyDescriptor.getName().equals("value")) {
                    propertyDescriptor.setValue("transient", Boolean.TRUE);
                }
            }
        }
        catch (IntrospectionException e) {
            throw new RuntimeException(e);
        }
    }

    private int speed = 0;

    private int trimDown = 0;

    private int trimUp = 0;

    private int value = 0;

    public int getSpeed() {
        return speed;
    }

    public void setSpeed(int speed) {
        this.speed = speed;
    }

    public int getTrimDown() {
        return trimDown;
    }

    public void setTrimDown(int trimDown) {
        this.trimDown = trimDown;
    }

    public int getTrimUp() {
        return trimUp;
    }

    public void setTrimUp(int trimUp) {
        this.trimUp = trimUp;
    }

    /**
     * Get the absolute servo destination value.
     * 
     * @return absolute servo destination value
     */
    public int getValue() {
        return value;
    }

    /**
     * Set the absolute servo destination value.
     * 
     * @param value
     *            absolute servo destination value
     */
    public void setValue(int value) {
        this.value = value;
    }

    /**
     * Get the absolute destination value from the relative value in %.
     * 
     * @param value
     *            relative servo destination value in %
     * 
     * @return absolute servo destination value
     */
    public int getAbsoluteValue(int value) {
        double stepWidth = (getTrimUp() - getTrimDown()) / 100D;

        return (int) (getTrimDown() + stepWidth * value);
    }

    /**
     * Get the relative servo destination value in % from the absolute value.
     * 
     * @param value
     *            absolute servo destination value
     * 
     * @return relative servo destination value in %
     */
    public int getRelativeValue(int value) {
        int result = 0;

        if (value >= getTrimDown() && value <= getTrimUp()) {
            double stepWidth = (getTrimUp() - getTrimDown()) / 100D;

            result = (int) ((value - getTrimDown()) / stepWidth);
        }
        return result;
    }

    /**
     * Set the relative servo destination value in %.
     * 
     * @param value
     *            relative servo destination value in %
     */
    public void setRelativeValue(int value) {
        this.value = getAbsoluteValue(value);
    }

    public String getDebugString() {
        return getClass().getSimpleName() + "[speed=" + speed + ",trimDown=" + trimDown + ",trimUp=" + trimUp
            + ",value=" + value + "]";
    }

    public static ServoPort load(String fileName) {
        ServoPort result = null;
        XMLDecoder d = null;

        try {
            try {
                d = new XMLDecoder(new GZIPInputStream(new BufferedInputStream(new FileInputStream(fileName))));

                result = (ServoPort) d.readObject();
            }
            catch (IOException e) {
                d = new XMLDecoder(new BufferedInputStream(new FileInputStream(fileName)));

                result = (ServoPort) d.readObject();
            }
        }
        catch (FileNotFoundException e) {
        }
        finally {
            if (d != null) {
                d.close();
            }
        }
        return result;
    }

    public void save(String fileName) throws IOException {
        XMLEncoder e = new XMLEncoder(new GZIPOutputStream(new BufferedOutputStream(new FileOutputStream(fileName))));

        e.writeObject(this);
        e.close();
    }
}
