package org.bidib.wizard.mvc.main.model;

import java.util.Collection;
import java.util.LinkedList;

import javax.swing.table.DefaultTableModel;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.listener.OutputListener;

public class SimplePortTableModel extends DefaultTableModel {
    private static final long serialVersionUID = 2035695990302627935L;

    private final Collection<OutputListener> portListeners = new LinkedList<OutputListener>();

    private String[] columnNames =
        { Resources.getString(getClass(), "label"), Resources.getString(getClass(), "test"), null };

    public SimplePortTableModel() {
        setColumnIdentifiers(columnNames);
    }

    public void addPortListener(OutputListener l) {
        portListeners.add(l);
    }

    public void addRow(Port port) {
        if (port != null) {
            Object[] rowData = new Object[getColumnCount()];

            rowData[0] = port.toString();
            rowData[1] = port.getStatus();
            rowData[2] = port;
            addRow(rowData);
        }
    }

    protected void fireLabelChanged(Port port, String value) {
        for (OutputListener l : portListeners) {
            l.labelChanged(port, value);
        }
    }

    private void fireTestButtonPressed(Port port) {
        for (OutputListener l : portListeners) {
            l.testButtonPressed(port);
        }
    }

    @Override
    public Class<?> getColumnClass(int column) {
        switch (column) {
            case 0:
                return String.class;
            default:
                return Object.class;
        }
    }

    @Override
    public void setValueAt(Object value, int row, int column) {
        final Object o = getValueAt(row, 2);

        if (o instanceof Port) {
            final Port port = (Port) o;

            switch (column) {
                case 0:
                    if (value instanceof String) {
                        port.setLabel((String) value);
                        super.setValueAt(port.toString(), row, column);
                        fireLabelChanged(port, port.getLabel());
                    }
                    else {
                        super.setValueAt(value, row, column);
                    }
                    break;
                case 1:
                    if (value instanceof Enum<?>) {
                        port.setStatus((Enum<?>) value);
                        super.setValueAt(value, row, column);
                        fireTestButtonPressed(port);
                    }
                    else {
                        super.setValueAt(value, row, column);
                    }
                    break;
                default:
                    super.setValueAt(value, row, column);
            }
        }
        else {
            super.setValueAt(value, row, column);
        }
    }
}
