package org.bidib.wizard.mvc.main.model;

import java.util.Collection;
import java.util.LinkedList;

import org.bidib.wizard.mvc.main.model.listener.StatusListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class StatusModel {
    private static final Logger LOGGER = LoggerFactory.getLogger(StatusModel.class);

    private final Collection<StatusListener> listeners = new LinkedList<StatusListener>();

    private boolean cd = false;

    private boolean running = true;

    private boolean rx = false;

    private boolean tx = false;

    public void addStatusListener(StatusListener l) {
        listeners.add(l);
    }

    /**
     * @return the port status connected
     */
    public boolean isCd() {
        return cd;
    }

    /**
     * @param cd the port status connected flag
     */
    public void setCd(boolean cd) {
        LOGGER.debug("setCd: {}", cd);
        this.cd = cd;
        fireCdChanged(cd);
    }

    public boolean isRunning() {
        return running;
    }

    public void setRunning(boolean running) {
        this.running = running;
        fireRunningChanged(running);
    }

    public boolean isRx() {
        return rx;
    }

    public void setRx(boolean rx) {
        LOGGER.trace("setRx: {}", rx);
        this.rx = rx;
        fireRxChanged(rx);
    }

    public boolean isTx() {
        return tx;
    }

    public void setTx(boolean tx) {
        LOGGER.trace("setTx: {}", tx);
        this.tx = tx;
        fireTxChanged(tx);
    }

    private void fireCdChanged(boolean cd) {
        for (StatusListener l : listeners) {
            l.cdChanged(cd);
        }
    }

    private void fireRunningChanged(boolean running) {
        for (StatusListener l : listeners) {
            l.runningChanged(running);
        }
    }

    private void fireRxChanged(boolean rx) {
        for (StatusListener l : listeners) {
            l.rxChanged(rx);
        }
    }

    private void fireTxChanged(boolean tx) {
        for (StatusListener l : listeners) {
            l.txChanged(tx);
        }
    }
}
