package org.bidib.wizard.mvc.main.view;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.WindowListener;

import javax.swing.event.ListSelectionListener;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.listener.LightPortListener;
import org.bidib.wizard.mvc.main.model.listener.OutputListener;
import org.bidib.wizard.mvc.main.model.listener.PortListener;
import org.bidib.wizard.mvc.main.model.listener.ServoPortListener;
import org.bidib.wizard.mvc.main.view.component.MyFrame;
import org.bidib.wizard.mvc.main.view.menu.MainMenuBar;
import org.bidib.wizard.mvc.main.view.menu.listener.MainMenuListener;
import org.bidib.wizard.mvc.main.view.panel.MainPanel;
import org.bidib.wizard.mvc.main.view.panel.listener.AccessoryListListener;
import org.bidib.wizard.mvc.main.view.panel.listener.AccessoryTableListener;
import org.bidib.wizard.mvc.main.view.panel.listener.MacroListListener;
import org.bidib.wizard.mvc.main.view.panel.listener.MacroTableListener;
import org.bidib.wizard.mvc.main.view.panel.listener.NodeListListener;
import org.bidib.wizard.mvc.main.view.panel.listener.StatusListener;

public class MainView extends MyFrame {
    private static final long serialVersionUID = -384371624920325232L;

    private final MainMenuBar mainMenuBar;

    private final MainPanel mainPanel;

    public MainView(MainModel model) {
        setLayout(new GridBagLayout());
        mainMenuBar = new MainMenuBar(model);
        setJMenuBar(mainMenuBar);
        setTitle(Resources.getString(getClass(), "title"));
        mainPanel = new MainPanel(model);

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.gridx = 0;
        c.gridy = 0;
        c.ipadx = 0;
        c.ipady = 0;
        c.weightx = 1;
        c.weighty = 1;
        c.insets = new Insets(5, 5, 5, 5);
        getContentPane().add(mainPanel, c);
    }

    public void addAccessoryListListener(AccessoryListListener l) {
        mainPanel.addAccessoryListListener(l);
    }

    public void addAccessoryListSelectionListener(ListSelectionListener l) {
        mainPanel.addAccessoryListSelectionListener(l);
    }

    public void addAccessoryTableListener(AccessoryTableListener l) {
        mainPanel.addAccessoryTableListener(l);
    }

    public void addAnalogPortListener(OutputListener l) {
        mainPanel.addAnalogPortListener(l);
    }

    public void addBoosterStatusListener(StatusListener l) {
        mainPanel.addBoosterStatusListener(l);
    }

    public void addInputPortListener(PortListener l) {
        mainPanel.addInputPortListener(l);
    }

    public void addLightPortListener(LightPortListener l) {
        mainPanel.addLightPortListener(l);
    }

    public void addMacroListListener(MacroListListener l) {
        mainPanel.addMacroListListener(l);
    }

    public void addMacroListSelectionListener(ListSelectionListener l) {
        mainPanel.addMacroListSelectionListener(l);
    }

    public void addMacroTableListener(MacroTableListener l) {
        mainPanel.addMacroTableListener(l);
    }

    public void addMainMenuListener(MainMenuListener l) {
        mainMenuBar.addMenuListener(l);
    }

    public void addMotorPortListener(OutputListener l) {
        mainPanel.addMotorPortListener(l);
    }

    public void addNodeListListener(NodeListListener l) {
        mainPanel.addNodeListListener(l);
    }

    public void addNodeListSelectionListener(ListSelectionListener l) {
        mainPanel.addNodeListSelectionListener(l);
    }

    public void addServoPortListener(ServoPortListener l) {
        mainPanel.addServoPortListener(l);
    }

    public void addSoundPortListener(OutputListener l) {
        mainPanel.addSoundPortListener(l);
    }

    public void addStatusListener(StatusListener l) {
        mainPanel.addStatusListener(l);
    }

    public void addSwitchPortListener(OutputListener l) {
        mainPanel.addSwitchPortListener(l);
    }

    public void setVisible(boolean visible) {
        pack();
        setMinimumSize(getPreferredSize());

        Dimension maxSize = Toolkit.getDefaultToolkit().getScreenSize();

        setSize(new Dimension(Math.min(maxSize.width, 1400), Math.min(maxSize.height, 800)));
        super.setVisible(visible);
    }

    public void setWindowListener(WindowListener l) {
        addWindowListener(l);
    }
}
