package org.bidib.wizard.mvc.main.view.menu;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import org.bidib.jbidibc.enumeration.IdentifyState;
import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.Node;
import org.bidib.wizard.mvc.main.view.menu.listener.NodeListMenuListener;

public class NodeListMenu {
    private static final Collection<NodeListMenuListener> menuListeners = new LinkedList<NodeListMenuListener>();

    private final MainModel model;

    private final JCheckBoxMenuItem identify = new JCheckBoxMenuItem(Resources.getString(getClass(), "identify"));

    private final JMenuItem firmwareUpdate = new JMenuItem(Resources.getString(getClass(), "firmwareUpdate") + " ...");

    private final JMenuItem loco = new JMenuItem(Resources.getString(getClass(), "loco") + " ...");

    private final JCheckBoxMenuItem addressMessagesEnabled =
        new JCheckBoxMenuItem(Resources.getString(getClass(), "addressMessages"));

    private final JCheckBoxMenuItem feedbackMessagesEnabled =
        new JCheckBoxMenuItem(Resources.getString(getClass(), "feedbackMessages"));

    private final JCheckBoxMenuItem keyMessagesEnabled =
        new JCheckBoxMenuItem(Resources.getString(getClass(), "keyMessages"));

    private final JCheckBoxMenuItem externalStartEnabled =
        new JCheckBoxMenuItem(Resources.getString(getClass(), "externalStart"));

    private final JCheckBoxMenuItem dccStartEnabled =
        new JCheckBoxMenuItem(Resources.getString(getClass(), "dccStart"));

    private final JMenuItem imp = new JMenuItem(Resources.getString(getClass(), "import") + " ...");

    private final JMenuItem export = new JMenuItem(Resources.getString(getClass(), "export") + " ...");

    private final JMenuItem save = new JMenuItem(Resources.getString(getClass(), "save"));

    public NodeListMenu(MainModel model) {
        this.model = model;
    }

    private void addMenuItem(Object menu, JMenuItem menuItem) {
        if (menu instanceof JMenu) {
            ((JMenu) menu).add(menuItem);
        }
        else if (menu instanceof JPopupMenu) {
            ((JPopupMenu) menu).add(menuItem);
        }
    }

    private void addNodeListMenu(Object menu) {
        JMenuItem editLabel = new JMenuItem(Resources.getString(getClass(), "editLabel") + " ...");

        editLabel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                fireEditLabel();
            }
        });
        addMenuItem(menu, editLabel);

        identify.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                fireIdentify(identify.isSelected());
            }
        });
        addMenuItem(menu, identify);

        JMenuItem details = new JMenuItem(Resources.getString(getClass(), "details") + " ...");

        details.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                fireDetails();
            }
        });
        addMenuItem(menu, details);

        JMenuItem cvProgrammer = new JMenuItem(Resources.getString(getClass(), "cvProgrammer") + " ...");

        cvProgrammer.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                fireCvProgrammer();
            }
        });
        addMenuItem(menu, cvProgrammer);

        firmwareUpdate.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireFirmwareUpdate();
            }
        });
        addMenuItem(menu, firmwareUpdate);

        loco.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireLoco();
            }
        });
        addMenuItem(menu, loco);

        addSeparator(menu);

        addressMessagesEnabled.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireAddressMessagesEnabled(addressMessagesEnabled.isSelected());
            }
        });
        addMenuItem(menu, addressMessagesEnabled);

        feedbackMessagesEnabled.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireFeedbackMessagesEnabled(feedbackMessagesEnabled.isSelected());
            }
        });
        addMenuItem(menu, feedbackMessagesEnabled);

        keyMessagesEnabled.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireKeyMessagesEnabled(keyMessagesEnabled.isSelected());
            }
        });
        addMenuItem(menu, keyMessagesEnabled);

        dccStartEnabled.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireDccStartEnabled(dccStartEnabled.isSelected());
            }
        });
        addMenuItem(menu, dccStartEnabled);

        externalStartEnabled.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireExternalStartEnabled(externalStartEnabled.isSelected());
            }
        });
        addMenuItem(menu, externalStartEnabled);

        addSeparator(menu);

        imp.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                fireImport();
            }
        });
        addMenuItem(menu, imp);

        export.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                fireExport();
            }
        });
        addMenuItem(menu, export);

        addSeparator(menu);

        save.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                fireSave();
            }
        });
        addMenuItem(menu, save);
    }

    public void addMenuListener(NodeListMenuListener l) {
        menuListeners.add(l);
    }

    private void addSeparator(Object menu) {
        if (menu instanceof JMenu) {
            ((JMenu) menu).addSeparator();
        }
        else if (menu instanceof JPopupMenu) {
            ((JPopupMenu) menu).addSeparator();
        }
    }

    private void fireAddressMessagesEnabled(boolean isSelected) {
        for (NodeListMenuListener l : menuListeners) {
            l.addressMessagesEnabled(isSelected);
        }
    }

    private void fireCvProgrammer() {
        for (NodeListMenuListener l : menuListeners) {
            l.cvProgrammer();
        }
    }

    private void fireDccStartEnabled(boolean isSelected) {
        for (NodeListMenuListener l : menuListeners) {
            l.dccStartEnabled(isSelected);
        }
    }

    private void fireDetails() {
        for (NodeListMenuListener l : menuListeners) {
            l.showDetails();
        }
    }

    private void fireEditLabel() {
        for (NodeListMenuListener l : menuListeners) {
            l.editLabel();
        }
    }

    private void fireExport() {
        for (NodeListMenuListener l : menuListeners) {
            l.exportNode();
        }
    }

    private void fireExternalStartEnabled(boolean isSelected) {
        for (NodeListMenuListener l : menuListeners) {
            l.externalStartEnabled(isSelected);
        }
    }

    private void fireFeedbackMessagesEnabled(boolean isSelected) {
        for (NodeListMenuListener l : menuListeners) {
            l.feedbackMessagesEnabled(isSelected);
        }
    }

    private void fireFirmwareUpdate() {
        for (NodeListMenuListener l : menuListeners) {
            l.firmwareUpdate();
        }
    }

    private void fireIdentify(boolean isSelected) {
        for (NodeListMenuListener l : menuListeners) {
            l.identify(isSelected);
        }
    }

    private void fireImport() {
        for (NodeListMenuListener l : menuListeners) {
            l.importNode();
        }
    }

    private void fireKeyMessagesEnabled(boolean isSelected) {
        for (NodeListMenuListener l : menuListeners) {
            l.keyMessagesEnabled(isSelected);
        }
    }

    private void fireLoco() {
        for (NodeListMenuListener l : menuListeners) {
            l.loco();
        }
    }

    private void fireSave() {
        for (NodeListMenuListener l : menuListeners) {
            l.saveNode();
        }
    }

    public JMenu getJMenu() {
        JMenu result = new JMenu();

        addNodeListMenu(result);
        return result;
    }

    public JPopupMenu getPopupMenu() {
        JPopupMenu result = new MyPopupMenu() {
            private static final long serialVersionUID = 5589102736959857382L;

            @Override
            public void show(Component invoker, int x, int y) {
                Node node = (Node) ((JList) invoker).getSelectedValue();

                firmwareUpdate.setEnabled(node.isUpdatable());
                loco.setEnabled(node.isCommandStation());
                identify.setSelected(node.getIdentifyState() == IdentifyState.START);

                boolean hasData = hasData();

                imp.setEnabled(hasData);
                export.setEnabled(hasData);
                save.setEnabled(hasData);
                super.show(invoker, x, y);
            }
        };

        addNodeListMenu(result);
        return result;
    }

    private boolean hasData() {
        return model.getAccessories().size() > 0 || model.getLightPorts().size() > 0 || model.getMacros().size() > 0
            || model.getServoPorts().size() > 0;
    }

    public void setAddressMessagesEnabled(Boolean isAddressMessagesEnabled) {
        addressMessagesEnabled.setVisible(isAddressMessagesEnabled != null);
        addressMessagesEnabled.setSelected(isAddressMessagesEnabled != null && isAddressMessagesEnabled);
    }

    public void setDccStartEnabled(Boolean isDccStartEnabled) {
        dccStartEnabled.setVisible(isDccStartEnabled != null);
        dccStartEnabled.setSelected(isDccStartEnabled != null && isDccStartEnabled);
    }

    public void setExternalStartEnabled(Boolean isExternalStartEnabled) {
        externalStartEnabled.setVisible(isExternalStartEnabled != null);
        externalStartEnabled.setSelected(isExternalStartEnabled != null && isExternalStartEnabled);
    }

    public void setFeedbackMessagesEnabled(Boolean isFeedbackMessagesEnabled) {
        feedbackMessagesEnabled.setVisible(isFeedbackMessagesEnabled != null);
        feedbackMessagesEnabled.setSelected(isFeedbackMessagesEnabled != null && isFeedbackMessagesEnabled);
    }

    public void setFirmwareUpdateEnabled(boolean isFirmwareUpdateEnabled) {
        firmwareUpdate.setEnabled(isFirmwareUpdateEnabled);
    }

    public void setIdentifyStateSelected(IdentifyState identifyState) {
        identify.setSelected(identifyState == IdentifyState.START);
    }

    public void setKeyMessagesEnabled(Boolean isKeyMessagesEnabled) {
        keyMessagesEnabled.setVisible(isKeyMessagesEnabled != null);
        keyMessagesEnabled.setSelected(isKeyMessagesEnabled != null && isKeyMessagesEnabled);
    }
}
