package org.bidib.wizard.mvc.main.view.panel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.bidib.wizard.dk.nullesoft.Airlog.LED;
import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.StatusModel;
import org.bidib.wizard.mvc.main.model.listener.StatusListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class LedPanel extends JPanel {
    private static final Logger LOGGER = LoggerFactory.getLogger(LedPanel.class);

    private static final long serialVersionUID = -1436881592908462890L;

    public LedPanel(StatusModel model) {
        setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Resources.getString(getClass(),
            "title")
            + ":"));
        setLayout(new GridBagLayout());

        final LED connectLed = new LED();
        connectLed.setName("CD");

        setLedStatus(connectLed, model.isCd());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.LINE_START;
        c.fill = GridBagConstraints.NONE;
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 0;
        c.weighty = 0;
        add(connectLed, c);

        final LED receiveLed = new LED();
        receiveLed.setName("RX");

        setLedStatus(receiveLed, model.isRx());

        c.gridx++;
        add(receiveLed, c);

        final LED sendLed = new LED();
        sendLed.setName("TX");

        setLedStatus(sendLed, model.isTx());

        c.gridx++;
        add(sendLed, c);

        c.gridx = 0;
        c.gridy++;
        add(new JLabel("CD"), c);

        c.gridx++;
        add(new JLabel("RX"), c);

        c.gridx++;
        add(new JLabel("TX"), c);

        model.addStatusListener(new StatusListener() {
            @Override
            public void cdChanged(boolean cd) {
                setLedStatus(connectLed, cd);
            }

            @Override
            public void runningChanged(boolean running) {
            }

            @Override
            public void rxChanged(boolean rx) {
                setLedStatus(receiveLed, rx);
            }

            @Override
            public void txChanged(boolean tx) {
                setLedStatus(sendLed, tx);
            }
        });
        setMinimumSize(getPreferredSize());
    }

    /**
     * Sets the led status. As this will call the swing components we must be sure
     * that the AWT-Thread is used to perform the update.
     * @param led the led
     * @param on the led status to set
     */
    private void setLedStatus(final LED led, final boolean on) {
        LOGGER.trace("setLedStatus led: {}, on: {}", led.getName(), on);
        if (SwingUtilities.isEventDispatchThread()) {
            if (on) {
                led.usePrimary();
            }
            else {
                led.useSecondary();
            }
        }
        else {
            SwingUtilities.invokeLater(new Runnable() {

                @Override
                public void run() {
                    if (on) {
                        led.usePrimary();
                    }
                    else {
                        led.useSecondary();
                    }
                }
            });
        }

    }
}
