package org.bidib.wizard.mvc.main.view.panel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;

import org.bidib.jbidibc.enumeration.IdentifyState;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.Node;
import org.bidib.wizard.mvc.main.view.menu.NodeListMenu;
import org.bidib.wizard.mvc.main.view.menu.listener.NodeListMenuListener;
import org.bidib.wizard.mvc.main.view.panel.listener.NodeListListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class NodeListPanel extends LabelListPanel implements NodeListMenuListener {
    private static final Logger LOGGER = LoggerFactory.getLogger(NodeListPanel.class);

    private static final long serialVersionUID = 5506124079986633240L;

    private final Collection<NodeListListener> nodeListListeners = new LinkedList<NodeListListener>();

    private final NodeListMenu nodeListMenu;

    private final JPopupMenu popupMenu;

    public NodeListPanel(final MainModel model) {
        nodeListMenu = new NodeListMenu(model);
        popupMenu = nodeListMenu.getPopupMenu();
        setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.weightx = 1;
        c.weighty = 1;
        c.insets = new Insets(5, 5, 5, 5);

        getLabelList().addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                int index = getLabelList().getIndex(e.getPoint());

                if (index >= 0) {
                    // get the selected node
                    Node node = ((Node) getLabelList().getModel().getElementAt(index));
                    LOGGER.debug("Node was selected: {}", node);

                    // update the menu items based on the features of the node
                    nodeListMenu.setAddressMessagesEnabled(node.isAddressMessagesEnabled());
                    nodeListMenu.setDccStartEnabled(node.isDccStartEnabled());
                    nodeListMenu.setExternalStartEnabled(node.isExternalStartEnabled());
                    nodeListMenu.setFeedbackMessagesEnabled(node.isFeedbackMessagesEnabled());
                    nodeListMenu.setKeyMessagesEnabled(node.isKeyMessagesEnabled());
                    handleMouseEvent(e, popupMenu);
                }
            }

            public void mouseReleased(MouseEvent e) {
                LOGGER.debug("Mouse released.");
                mousePressed(e);
            }
        });
        getLabelList().setListData(model.getNodes().toArray());
        getLabelList().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        model.addNodeListListener(new org.bidib.wizard.mvc.main.model.listener.NodeListListener() {
            @Override
            public void listChanged() {
                LOGGER.debug("List changed, use the AWT-thread to set the nodes in the label list.");
                if (SwingUtilities.isEventDispatchThread()) {
                    internalListChanged(model);
                }
                else {
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            internalListChanged(model);
                        }
                    });
                }

            }

            @Override
            public void nodeChanged() {
            }
        });
        nodeListMenu.addMenuListener(this);
        add(new JScrollPane(getLabelList()), c);
    }

    private void internalListChanged(final MainModel model) {
        Object selectedValue = getLabelList().getSelectedValue();
        LOGGER.debug("The list has changed, current selected value: {}", selectedValue);
        getLabelList().setListData(model.getNodes().toArray());
        LOGGER.debug("The list has changed, set the new nodes has finished.");

        // TODO we should set the previous selected value but this does not work currently
        //        if (selectedValue != null && model.getNodes().contains(selectedValue)) {
        //            LOGGER.debug("Set the previous selected value: {}", selectedValue);
        //            getLabelList().setSelectedValue(selectedValue, true);
        //        }
    }

    @Override
    public void addressMessagesEnabled(boolean isSelected) {
        Node node = (Node) getLabelList().getSelectedValue();

        if (node != null) {
            node.setAddressMessagesEnabled(isSelected);
            for (NodeListListener l : nodeListListeners) {
                l.enableAddressMessages(node);
            }
        }
    }

    public void addNodeListListener(NodeListListener l) {
        addLabelChangedListener(l);
        nodeListListeners.add(l);
    }

    @Override
    public void cvProgrammer() {
        Point itemPosition = selectedIndexToLocation();

        for (NodeListListener l : nodeListListeners) {
            l.cvProgrammer((Node) getLabelList().getSelectedValue(), itemPosition.x, itemPosition.y);
        }
    }

    @Override
    public void dccStartEnabled(boolean isSelected) {
        Node node = (Node) getLabelList().getSelectedValue();

        if (node != null) {
            node.setDccStartEnabled(isSelected);
            for (NodeListListener l : nodeListListeners) {
                l.enableDccStart(node);
            }
        }
    }

    @Override
    public void exportNode() {
        Node node = (Node) getLabelList().getSelectedValue();

        if (node != null) {
            for (NodeListListener l : nodeListListeners) {
                l.exportNode(node);
            }
        }
    }

    @Override
    public void externalStartEnabled(boolean isSelected) {
        Node node = (Node) getLabelList().getSelectedValue();

        if (node != null) {
            node.setExternalStartEnabled(isSelected);
            for (NodeListListener l : nodeListListeners) {
                l.enableExternalStart(node);
            }
        }
    }

    @Override
    public void feedbackMessagesEnabled(boolean isSelected) {
        Node node = (Node) getLabelList().getSelectedValue();

        if (node != null) {
            node.setFeedbackMessagesEnabled(isSelected);
            for (NodeListListener l : nodeListListeners) {
                l.enableFeedbackMessages(node);
            }
        }
    }

    @Override
    public void firmwareUpdate() {
        Point itemPosition = selectedIndexToLocation();

        for (NodeListListener l : nodeListListeners) {
            l.firmwareUpdate((Node) getLabelList().getSelectedValue(), itemPosition.x, itemPosition.y);
        }
    }

    @Override
    public void identify(boolean isSelected) {
        Node node = (Node) getLabelList().getSelectedValue();

        if (node != null) {
            node.setIdentifyState(isSelected ? IdentifyState.START : IdentifyState.STOP);
            for (NodeListListener l : nodeListListeners) {
                l.identify(node);
            }
        }
    }

    @Override
    public void importNode() {
        Node node = (Node) getLabelList().getSelectedValue();

        if (node != null) {
            for (NodeListListener l : nodeListListeners) {
                l.importNode(node);
            }
        }
    }

    @Override
    public void keyMessagesEnabled(boolean isSelected) {
        Node node = (Node) getLabelList().getSelectedValue();

        if (node != null) {
            node.setKeyMessagesEnabled(isSelected);
            for (NodeListListener l : nodeListListeners) {
                l.enableKeyMessages(node);
            }
        }
    }

    @Override
    public void loco() {
        Point itemPosition = selectedIndexToLocation();
        Node node = (Node) getLabelList().getSelectedValue();

        if (node != null) {
            for (NodeListListener l : nodeListListeners) {
                l.loco(node, itemPosition.x, itemPosition.y);
            }
        }
    }

    @Override
    public void saveNode() {
        Node node = (Node) getLabelList().getSelectedValue();

        if (node != null) {
            for (NodeListListener l : nodeListListeners) {
                l.saveNode(node);
            }
        }
    }

    @Override
    public void showDetails() {
        Point itemPosition = selectedIndexToLocation();

        for (NodeListListener l : nodeListListeners) {
            l.nodeDetails((Node) getLabelList().getSelectedValue(), itemPosition.x, itemPosition.y);
        }
    }
}
