package org.bidib.wizard.mvc.main.view.panel;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JToggleButton;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.StatusModel;
import org.bidib.wizard.mvc.main.view.panel.listener.StatusListener;

public class StatusButtonPanel extends JPanel {
    private static final long serialVersionUID = -3975992888994980016L;

    private Collection<StatusListener> statusListeners = new LinkedList<StatusListener>();

    public StatusButtonPanel(StatusModel model) {
        setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Resources.getString(getClass(),
            "title")
            + ":"));
        setLayout(new GridBagLayout());

        final JToggleButton onButton = new JToggleButton();

        onButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JToggleButton button = (JToggleButton) e.getSource();

                if (button.isSelected()) {
                    fireSwitchedOn();
                }
                else {
                    fireSwitchedOff();
                }
            }
        });
        onButton.setSelected(model.isRunning());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.CENTER;
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.insets = new Insets(5, 5, 5, 5);
        add(onButton, c);

        if (model.isRunning()) {
            onButton.setText(Resources.getString(getClass(), "on"));
        }
        else {
            onButton.setText(Resources.getString(getClass(), "off"));
        }

        Dimension size = onButton.getPreferredSize();

        onButton.setPreferredSize(new Dimension(60, size.height));

        model.addStatusListener(new org.bidib.wizard.mvc.main.model.listener.StatusListener() {
            @Override
            public void cdChanged(boolean cd) {
            }

            @Override
            public void runningChanged(boolean running) {
                if (running) {
                    onButton.setText(Resources.getString(StatusButtonPanel.class, "on"));
                }
                else {
                    onButton.setText(Resources.getString(StatusButtonPanel.class, "off"));
                }
            }

            @Override
            public void rxChanged(boolean rx) {
            }

            @Override
            public void txChanged(boolean tx) {
            }
        });
    }

    public void addStatusListener(StatusListener l) {
        statusListeners.add(l);
    }

    private void fireSwitchedOff() {
        for (StatusListener l : statusListeners) {
            l.switchedOff();
        }
    }

    private void fireSwitchedOn() {
        for (StatusListener l : statusListeners) {
            l.switchedOn();
        }
    }
}
