package org.bidib.wizard.mvc.main.view.panel;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionListener;

import org.bidib.jbidibc.enumeration.IdentifyState;
import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.Node;
import org.bidib.wizard.mvc.main.model.listener.LightPortListener;
import org.bidib.wizard.mvc.main.model.listener.NodeListListener;
import org.bidib.wizard.mvc.main.model.listener.NodeListener;
import org.bidib.wizard.mvc.main.model.listener.OutputListener;
import org.bidib.wizard.mvc.main.model.listener.PortListListener;
import org.bidib.wizard.mvc.main.model.listener.PortListener;
import org.bidib.wizard.mvc.main.model.listener.ServoPortListener;
import org.bidib.wizard.mvc.main.view.panel.listener.AccessoryListListener;
import org.bidib.wizard.mvc.main.view.panel.listener.AccessoryTableListener;
import org.bidib.wizard.mvc.main.view.panel.listener.MacroListListener;
import org.bidib.wizard.mvc.main.view.panel.listener.MacroTableListener;
import org.bidib.wizard.mvc.main.view.panel.listener.StatusListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class TabPanel extends JPanel implements NodeListener, NodeListListener {
    private static final long serialVersionUID = 5506124079986633240L;

    private static final Logger LOGGER = LoggerFactory.getLogger(TabPanel.class);

    private String emptyLabel;

    private final MainModel model;

    private TitledBorder border;

    private final AccessoryListPanel accessoryListPanel;

    private final AnalogPortListPanel analogPortListPanel;

    private final BoosterPanel boosterPanel;

    private final FeedbackPortListPanel feedbackPortListPanel;

    private final InputPortListPanel inputPortListPanel;

    private final LightPortListPanel lightPortListPanel;

    private final MacroListPanel macroListPanel;

    private final MotorPortListPanel motorPortListPanel;

    private final ServoPortListPanel servoPortListPanel;

    private final SoundPortListPanel soundPortListPanel;

    private final SwitchPortListPanel switchPortListPanel;

    private final JTabbedPane tabbedPane;

    private Node node = null;

    private final Collection<JPanel> tabs = new LinkedList<JPanel>();

    private EmptyPanel emptyPanel;

    public TabPanel(final MainModel model) {
        this.model = model;
        emptyLabel = Resources.getString(getClass(), "emptyLabel");
        border = BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), emptyLabel);

        tabbedPane = new JTabbedPane();
        emptyPanel = new EmptyPanel();

        setBorder(border);
        setLayout(new GridBagLayout());

        accessoryListPanel = new AccessoryListPanel(model);
        analogPortListPanel = new AnalogPortListPanel(model);
        boosterPanel = new BoosterPanel(model);
        feedbackPortListPanel = new FeedbackPortListPanel(model);
        inputPortListPanel = new InputPortListPanel(model);
        lightPortListPanel = new LightPortListPanel(model);
        macroListPanel = new MacroListPanel(model);
        motorPortListPanel = new MotorPortListPanel(model);
        servoPortListPanel = new ServoPortListPanel(model);
        soundPortListPanel = new SoundPortListPanel(model);
        switchPortListPanel = new SwitchPortListPanel(model);

        tabs.add(macroListPanel);
        tabs.add(accessoryListPanel);
        tabs.add(inputPortListPanel);
        tabs.add(analogPortListPanel);
        tabs.add(lightPortListPanel);
        tabs.add(motorPortListPanel);
        tabs.add(switchPortListPanel);
        tabs.add(servoPortListPanel);
        tabs.add(soundPortListPanel);
        tabs.add(boosterPanel);
        tabs.add(feedbackPortListPanel);

        showInnerComponent(node == null);

        model.addNodeListListener(this);

        model.addAccessoryListListener(new org.bidib.wizard.mvc.main.model.listener.AccessoryListListener() {
            @Override
            public void listChanged() {
                enableTab(node, accessoryListPanel, !model.getAccessories().isEmpty());
            }

            @Override
            public void accessoryChanged() {
            }
        });
        model.addAnalogPortListListener(new PortListListener() {
            @Override
            public void listChanged() {
                enableTab(node, analogPortListPanel, !model.getAnalogPorts().isEmpty());
            }
        });
        model.addFeedbackPortListListener(new PortListListener() {
            @Override
            public void listChanged() {
                boolean feedbackPortsAvailable = !model.getFeedbackPorts().isEmpty();
                LOGGER.debug("change visibility of feedbackPortListPanel, feedbackPortsAvailable: {}",
                    feedbackPortsAvailable);

                enableTab(node, feedbackPortListPanel, feedbackPortsAvailable/*!model.getFeedbackPorts().isEmpty()*/);
            }
        });
        model.addInputPortListListener(new PortListListener() {
            @Override
            public void listChanged() {
                enableTab(node, inputPortListPanel, !model.getInputPorts().isEmpty());
            }
        });
        model.addLightPortListListener(new PortListListener() {
            @Override
            public void listChanged() {
                LOGGER.debug("listChanged in LightPortListener, model.lightPorts: {}", model.getLightPorts());
                enableTab(node, lightPortListPanel, !model.getLightPorts().isEmpty());
            }
        });
        model.addMacroListListener(new org.bidib.wizard.mvc.main.model.listener.MacroListListener() {
            @Override
            public void listChanged() {
                enableTab(node, macroListPanel, !model.getMacros().isEmpty());
            }

            @Override
            public void macroChanged() {
            }
        });
        model.addMotorPortListListener(new PortListListener() {
            @Override
            public void listChanged() {
                enableTab(node, motorPortListPanel, !model.getMotorPorts().isEmpty());
            }
        });
        model.addServoPortListListener(new PortListListener() {
            @Override
            public void listChanged() {
                enableTab(node, servoPortListPanel, !model.getServoPorts().isEmpty());
            }
        });
        model.addSoundPortListListener(new PortListListener() {
            @Override
            public void listChanged() {
                enableTab(node, soundPortListPanel, !model.getSoundPorts().isEmpty());
            }
        });
        model.addSwitchPortListListener(new PortListListener() {
            @Override
            public void listChanged() {
                enableTab(node, switchPortListPanel, !model.getSwitchPorts().isEmpty());
            }
        });

    }

    private void showInnerComponent(boolean empty) {

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.weightx = 1;
        c.weighty = 1;
        c.insets = new Insets(5, 5, 5, 5);

        if (empty) {
            remove(tabbedPane);
            add(emptyPanel, c);
        }
        else {
            remove(emptyPanel);
            add(tabbedPane, c);
        }
    }

    public void addAccessoryListListener(AccessoryListListener l) {
        accessoryListPanel.addAccessoryListListener(l);
    }

    public void addAccessoryListSelectionListener(ListSelectionListener l) {
        accessoryListPanel.addListSelectionListener(l);
    }

    public void addAccessoryTableListener(AccessoryTableListener l) {
        accessoryListPanel.addAccessoryTableListener(l);
    }

    public void addAnalogPortListener(OutputListener l) {
        analogPortListPanel.addPortListener(l);
    }

    public void addBoosterStatusListener(StatusListener l) {
        boosterPanel.addStatusListener(l);
    }

    public void addInputPortListener(PortListener l) {
        inputPortListPanel.addPortListener(l);
    }

    public void addLightPortListener(LightPortListener l) {
        lightPortListPanel.addLightPortListener(l);
    }

    public void addMacroListListener(MacroListListener l) {
        macroListPanel.addMacroListListener(l);
    }

    public void addMacroListSelectionListener(ListSelectionListener l) {
        macroListPanel.addListSelectionListener(l);
    }

    public void addMacroTableListener(MacroTableListener l) {
        macroListPanel.addMacroTableListener(l);
    }

    public void addMotorPortListener(OutputListener l) {
        motorPortListPanel.addPortListener(l);
    }

    public void addServoPortListener(ServoPortListener l) {
        servoPortListPanel.addServoPortListener(l);
    }

    public void addSoundPortListener(OutputListener l) {
        soundPortListPanel.addPortListener(l);
    }

    public void addSwitchPortListener(OutputListener l) {
        switchPortListPanel.addPortListener(l);
    }

    @Override
    public void addressMessagesEnabledChanged(Boolean isAddressMessagesEnabled) {
    }

    @Override
    public void dccStartEnabledChanged(Boolean isDccStartEnabled) {
    }

    @Override
    public void externalStartEnabledChanged(Boolean isExternalStartEnabled) {
    }

    @Override
    public void feedbackMessagesEnabledChanged(Boolean isFeedbackMessagesEnabled) {
    }

    @Override
    public void identifyStateChanged(IdentifyState identifyState) {
    }

    @Override
    public void keyMessagesEnabledChanged(Boolean isKeyMessagesEnabled) {
    }

    @Override
    public void labelChanged(String label) {
        if (!label.equals(emptyLabel)) {
            label = node.toString();
        }
        LOGGER.debug("Change the tab label: {}", label);
        if (SwingUtilities.isEventDispatchThread()) {
            border.setTitle(label + (label.equals(emptyLabel) ? "" : ":"));
            repaint();
        }
        else {
            final String title = label;
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    border.setTitle(title + (title.equals(emptyLabel) ? "" : ":"));
                    repaint();
                }
            });
        }
    }

    @Override
    public void listChanged() {
        LOGGER.debug("list has changed.");
    }

    @Override
    public void nodeChanged() {
        LOGGER.debug("The node has changed: {}", node);
        if (SwingUtilities.isEventDispatchThread()) {
            internalNodeChanged();
        }
        else {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    internalNodeChanged();
                }
            });
        }
    }

    private void internalNodeChanged() {
        LOGGER.debug("handle node has changed: {}", node);

        if (node != null && node.equals(model.getNode())) {
            LOGGER.debug("The node has not changed.");
            return;
        }

        if (node != null) {
            node.removeNodeListener(this);
        }
        // remove all tabs because otherwise while loading featues from LC the booster panel is still displayed
        tabbedPane.removeAll();

        node = model.getNode();
        showInnerComponent(node == null);

        // the booster panel needs special handling because it has no list of ports
        enableTab(node, boosterPanel, node != null && node.getNode().hasBoosterFunctions());

        if (node != null) {
            node.addNodeListener(this);
            labelChanged(node.toString());
        }
        else {
            labelChanged(emptyLabel);
        }
    }

    private void enableTab(final Node node, final Component component, final boolean enable) {
        if (SwingUtilities.isEventDispatchThread()) {
            innerEnableTab(node, component, enable);
        }
        else {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    innerEnableTab(node, component, enable);
                }
            });
        }
    }

    private void innerEnableTab(final Node node, final Component component, final boolean enable) {

        int componentIndex = tabbedPane.indexOfComponent(component);
        LOGGER.debug("enableTab, enable: {}, componentIndex: {}, component: {}", enable, componentIndex, component);
        if (enable && componentIndex == -1) {

            // count all visible components in front of me
            int index = 0;

            for (Component tab : tabs) {
                if (tab == component) {
                    break;
                }
                if (tabbedPane.indexOfComponent(tab) >= 0) {
                    index++;
                }
            }
            tabbedPane.add(component, index);
            if (index == 0) {
                tabbedPane.setSelectedIndex(0);
            }
        }
        else if (!enable && componentIndex >= 0) {
            tabbedPane.remove(component);
        }
        LOGGER.debug("number of components in tabbed pane: {}", tabbedPane.getComponentCount());
    }
}
