package org.bidib.wizard.mvc.main.view.table;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Collection;

import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;

import org.bidib.jbidibc.enumeration.AddressTypeEnum;
import org.bidib.wizard.comm.FeedbackPortStatus;
import org.bidib.wizard.mvc.common.view.panel.DisabledPanel;
import org.bidib.wizard.mvc.main.model.AddressData;
import org.bidib.wizard.mvc.main.model.FeedbackPort;

public class FeedbackPortTableCellRenderer extends InputPortTableCellRenderer {
    private static final long serialVersionUID = 9064420671442830649L;

    private String getAddressData(AddressData address) {
        StringBuffer result = new StringBuffer();

        if (address != null) {
            if (result.length() > 0) {
                result.append(',');
            }
            result.append(address.getAddress());
            if (address.getType() == AddressTypeEnum.LOCOMOTIVE_FORWARD) {
                result.append(">>");
            }
            else if (address.getType() == AddressTypeEnum.LOCOMOTIVE_BACKWARD) {
                result.append("<<");
            }
            result.append('(');
            result.append(address.getSpeed());
            result.append(')');
        }
        else {
            result.append("[]");
        }
        return result.toString();
    }

    public Component getTableCellRendererComponent(
        JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
        JPanel result = new JPanel(new GridBagLayout());
        JPanel innerPanel = new JPanel(new GridBagLayout());
        FeedbackPort port = (FeedbackPort) value;
        Collection<AddressData> addresses = port.getAddresses();
        FeedbackPortStatus status = (FeedbackPortStatus) port.getStatus();
        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.CENTER;
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1;

        JLabel portLabel = new JLabel("Port " + port.getId() + ":");
        Font f = portLabel.getFont();

        portLabel.setFont(f.deriveFont(f.getStyle() | Font.BOLD));
        innerPanel.add(portLabel, c);

        c.gridy++;
        innerPanel.add(new JLabel((port.isValid() ? "1" : "0") + "," + (port.isFreeze() ? "1" : "0") + ","
            + (port.isSignal() ? "1" : "0")), c);

        if (addresses != null) {
            for (AddressData address : addresses) {
                c.gridy++;
                innerPanel.add(new JLabel(getAddressData(address)), c);
            }
        }

        c.fill = GridBagConstraints.BOTH;
        c.gridy++;
        c.weighty = 1;
        innerPanel.add(Box.createVerticalGlue(), c);

        // background color
        if (status == FeedbackPortStatus.OCCUPIED) {
            innerPanel.setBackground(Color.RED);
        }
        else {
            innerPanel.setBackground(Color.GREEN);
        }

        // gray out depending on confidence
        DisabledPanel disabledInnerPanel = new DisabledPanel(innerPanel);

        c.gridy = 0;
        result.add(disabledInnerPanel, c);
        disabledInnerPanel.setEnabled(!(port.isFreeze() || port.isSignal() || port.isValid()));

        return result;
    }
}
