package org.bidib.wizard.mvc.main.view.table;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.AbstractCellEditor;
import javax.swing.JButton;
import javax.swing.JFormattedTextField;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.TableCellEditor;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.NumberFormatter;

import org.bidib.wizard.mvc.main.view.table.listener.ButtonListener;

public class NumberWithButtonEditor extends AbstractCellEditor implements TableCellEditor {
    private static final long serialVersionUID = -5074223597210117377L;

    private final Collection<ButtonListener> buttonListeners = new LinkedList<ButtonListener>();

    private final JPanel panel = new JPanel();

    private final JFormattedTextField textField = new JFormattedTextField();

    private final NumberFormatter formatter = new NumberFormatter(
        new DecimalFormat("#")) {
        private static final long serialVersionUID = 9071735589476805065L;

        {
            setAllowsInvalid(false);
            setMinimum(0);
        }
    };

    private final JButton button;

    private int row = 0;

    private int column = 0;

    public NumberWithButtonEditor(String buttonText, int maximum) {
        formatter.setMaximum(maximum);
        textField.setFormatterFactory(new DefaultFormatterFactory(formatter, formatter, formatter));
        textField.setHorizontalAlignment(SwingConstants.RIGHT);
        button = new JButton(buttonText);
        button.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireEditingStopped();
                fireButtonPressed();
            }
        });
        panel.setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.insets = new Insets(0, 0, 2, 2);
        c.weightx = 1;
        panel.add(textField, c);
        c.anchor = GridBagConstraints.FIRST_LINE_END;
        c.gridx++;
        c.weightx = 0;
        panel.add(button, c);
    }

    public NumberWithButtonEditor(String buttonText) {
        this(buttonText, 255);
    }

    public void addButtonListener(ButtonListener l) {
        buttonListeners.add(l);
    }

    private void fireButtonPressed() {
        for (ButtonListener l : buttonListeners) {
            l.buttonPressed(row, column);
        }
    }

    @Override
    public Object getCellEditorValue() {
        return Integer.parseInt(textField.getText());
    }

    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
        this.row = row;
        this.column = column;
        if (isSelected) {
            panel.setForeground(table.getSelectionForeground());
            panel.setBackground(table.getSelectionBackground());
        }
        else {
            panel.setForeground(table.getForeground());
            panel.setBackground(table.getBackground());
        }
        textField.setText(value != null ? value.toString() : "");
        return panel;
    }

    @Override
    public boolean stopCellEditing() {
        try {
            int v = Integer.valueOf(textField.getText());
            if (v < 0) {
                throw new NumberFormatException();
            }
        }
        catch (NumberFormatException e) {
            return false;
        }
        return super.stopCellEditing();
    }
}
