package org.bidib.wizard.mvc.main.view.table;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.AbstractCellEditor;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

public class SliderEditor extends AbstractCellEditor implements TableCellEditor, TableCellRenderer {
    private static final long serialVersionUID = 4672003636854301184L;

    private final JPanel panel = new JPanel(new GridBagLayout());

    private final JSlider slider = new JSlider(0, 255);

    private final JLabel sliderValue = new JLabel();

    public SliderEditor() {
        slider.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                JSlider source = (JSlider) e.getSource();

                sliderValue.setText(String.valueOf(source.getValue()));
                if (!source.getValueIsAdjusting()) {
                    stopCellEditing();
                }
            }
        });
        sliderValue.setText("0");

        Dimension d = sliderValue.getPreferredSize();

        d.width += 20;
        sliderValue.setMaximumSize(d);
        sliderValue.setMinimumSize(d);

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.LINE_START;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridwidth = GridBagConstraints.RELATIVE;
        c.insets = new Insets(0, 5, 0, 5);
        c.weightx = 0.9;
        c.weighty = 1;
        panel.add(slider, c);

        c.anchor = GridBagConstraints.LINE_END;
        c.fill = GridBagConstraints.NONE;
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.weightx = 0.1;
        panel.add(sliderValue, c);
    }

    @Override
    public Object getCellEditorValue() {
        return slider.getValue();
    }

    @Override
    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
        if (isSelected) {
            slider.setForeground(table.getSelectionForeground());
            slider.setBackground(table.getSelectionBackground());
        }
        else {
            slider.setForeground(table.getForeground());
            slider.setBackground(table.getBackground());
        }
        slider.setValue((Integer) value);
        return panel;
    }

    @Override
    public Component getTableCellRendererComponent(
        JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
        if (isSelected) {
            slider.setForeground(table.getSelectionForeground());
            slider.setBackground(table.getSelectionBackground());
        }
        else {
            slider.setForeground(table.getForeground());
            slider.setBackground(UIManager.getColor("Button.background"));
        }
        slider.setValue((Integer) value);
        slider.updateUI();
        return panel;
    }

    public void setDown() {
        slider.setValue(slider.getValue() - 1);
    }

    public void setUp() {
        slider.setValue(slider.getValue() + 1);
    }
}
