package org.bidib.wizard.mvc.preferences.controller;

import gnu.io.CommPortIdentifier;

import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.List;

import javax.swing.JFrame;

import org.bidib.jbidibc.Bidib;
import org.bidib.wizard.mvc.preferences.model.CommPort;
import org.bidib.wizard.mvc.preferences.model.Preferences;
import org.bidib.wizard.mvc.preferences.model.PreferencesModel;
import org.bidib.wizard.mvc.preferences.view.PreferencesView;
import org.bidib.wizard.mvc.preferences.view.listener.PreferencesViewListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class PreferencesController {
    private static final Logger LOGGER = LoggerFactory.getLogger(PreferencesController.class);

    private final JFrame parent;

    private final PreferencesModel model = new PreferencesModel();

    public PreferencesController(JFrame parent) {
        this.parent = parent;
    }

    private void loadCommPorts() {
        if (!Preferences.getInstance().isMock()) {
            LOGGER.info("Load the comm ports.");
            Collection<CommPort> commPorts = new HashSet<CommPort>();

            // use Bidib because we must load the RXTX libraries
            List<CommPortIdentifier> portIdentifiers = Bidib.getPortIdentifiers();

            if (portIdentifiers != null) {
                for (CommPortIdentifier id : portIdentifiers) {
                    if (id.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                        commPorts.add(new CommPort(id));
                    }
                }
            }
            model.setCommPorts(commPorts);
        }
    }

    public void start() {
        loadCommPorts();

        PreferencesView view = new PreferencesView(parent, model);

        view.addPreferencesViewListener(new PreferencesViewListener() {
            @Override
            public void commPortChanged(CommPort port) {
                LOGGER.info("The comm port has changed: {}", port);

                // reconnect is triggered
                Preferences.getInstance().setCommPort(port != null ? port.toString() : "");

            }

            @Override
            public void mockChanged(boolean mock) {
                Preferences.getInstance().setMock(mock);
                loadCommPorts();
            }

            @Override
            public void save() {
                Preferences.getInstance().save();

                // reload the logger configuration because the path to the log file might have changed 
                Preferences.getInstance().reloadLoggerConfiguration();
            }

            @Override
            public void startTimeChanged(Date startTime) {
                Preferences.getInstance().setStartTime(startTime);
            }

            @Override
            public void timeFactorChanged(int timeFactor) {
                Preferences.getInstance().setTimeFactor(timeFactor);
            }
        });
    }
}
