package org.bidib.wizard.mvc.preferences.view;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.preferences.model.PreferencesModel;
import org.bidib.wizard.mvc.preferences.view.listener.PreferencesViewListener;
import org.bidib.wizard.mvc.preferences.view.panel.MiscPanel;
import org.bidib.wizard.mvc.preferences.view.panel.TimePanel;

public class PreferencesView extends JDialog {
    private static final long serialVersionUID = -3652775401908609757L;

    private final Collection<PreferencesViewListener> listeners = new LinkedList<PreferencesViewListener>();

    private final MiscPanel miscPanel;

    private final TimePanel timePanel;

    public PreferencesView(JFrame parent, PreferencesModel model) {
        super(parent);
        setTitle(Resources.getString(getClass(), "title"));

        JPanel mainPanel = new JPanel(new GridBagLayout());

        miscPanel = new MiscPanel(model);

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1;
        c.insets = new Insets(5, 5, 5, 5);
        mainPanel.add(miscPanel, c);

        timePanel = new TimePanel();
        c.gridx++;
        c.weightx = 0;
        c.weighty = 1;
        mainPanel.add(timePanel, c);

        JButton save = new JButton(Resources.getString(getClass(), "save"));

        save.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
                fireSave();
            }
        });
        c.anchor = GridBagConstraints.LAST_LINE_END;
        c.fill = GridBagConstraints.NONE;
        c.gridx = 1;
        c.gridy++;
        c.weighty = 0;
        mainPanel.add(save, c);

        getContentPane().add(mainPanel);
        pack();
        setLocationRelativeTo(parent);
        setMinimumSize(getSize());
        setVisible(true);
    }

    public void addPreferencesViewListener(PreferencesViewListener l) {
        listeners.add(l);
        miscPanel.addPreferencesViewListener(l);
        timePanel.addPreferencesViewListener(l);
    }

    private void fireSave() {
        for (PreferencesViewListener l : listeners) {
            l.save();
        }
    }
}
