package org.bidib.wizard.mvc.preferences.view.panel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.preferences.model.CommPort;
import org.bidib.wizard.mvc.preferences.model.Preferences;
import org.bidib.wizard.mvc.preferences.model.PreferencesModel;
import org.bidib.wizard.mvc.preferences.model.listener.PreferencesModelListener;
import org.bidib.wizard.mvc.preferences.view.listener.PreferencesViewListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MiscPanel extends JPanel {
    private static final Logger LOGGER = LoggerFactory.getLogger(MiscPanel.class);

    private static final long serialVersionUID = -5927872456501821169L;

    private final Collection<PreferencesViewListener> listeners = new LinkedList<PreferencesViewListener>();

    private final PreferencesModel model;

    public MiscPanel(PreferencesModel model) {
        this.model = model;
        setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Resources.getString(getClass(),
            "title")
            + ":"));
        setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 0;
        c.weighty = 0;
        c.insets = new Insets(5, 5, 5, 5);
        add(new JLabel(Resources.getString(getClass(), "serialPort") + ":"), c);

        final JComboBox commPorts = new JComboBox();

        commPorts.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireCommPortChanged((CommPort) commPorts.getSelectedItem());
            }
        });
        commPorts.setEnabled(!Preferences.getInstance().isMock());
        setCommPorts(commPorts);
        c.gridx += 1;
        c.weightx = 1;
        add(commPorts, c);

        c.gridx = 0;
        c.gridy += 1;
        c.weightx = 0;
        add(new JLabel(Resources.getString(getClass(), "labelPath") + ":"), c);

        final JTextField labelPath = new JTextField();

        labelPath.getDocument().addDocumentListener(new DocumentListener() {
            public void changedUpdate(DocumentEvent e) {
                setText();
            }

            public void removeUpdate(DocumentEvent e) {
                setText();
            }

            public void insertUpdate(DocumentEvent e) {
                setText();
            }

            private void setText() {
                Preferences.getInstance().setLabelPath(labelPath.getText());
            }
        });
        labelPath.setText(Preferences.getInstance().getLabelPath());
        c.gridx += 1;
        c.weightx = 1;
        add(labelPath, c);

        c.gridx = 0;
        c.gridy += 1;
        c.weightx = 0;
        add(new JLabel(Resources.getString(getClass(), "logFile") + ":"), c);

        final JTextField logFile = new JTextField();

        logFile.getDocument().addDocumentListener(new DocumentListener() {
            public void changedUpdate(DocumentEvent e) {
                setText();
            }

            public void removeUpdate(DocumentEvent e) {
                setText();
            }

            public void insertUpdate(DocumentEvent e) {
                setText();
            }

            private void setText() {
                Preferences.getInstance().setLogFile(logFile.getText());
            }
        });
        logFile.setText(Preferences.getInstance().getLogFile());
        c.gridx += 1;
        c.weightx = 1;
        add(logFile, c);

        c.gridx = 0;
        c.gridy += 1;
        c.weightx = 0;
        add(new JLabel(Resources.getString(getClass(), "rxtxLogFile") + ":"), c);

        final JTextField rxtxLogFile = new JTextField();

        rxtxLogFile.getDocument().addDocumentListener(new DocumentListener() {
            public void changedUpdate(DocumentEvent e) {
                setText();
            }

            public void removeUpdate(DocumentEvent e) {
                setText();
            }

            public void insertUpdate(DocumentEvent e) {
                setText();
            }

            private void setText() {
                Preferences.getInstance().setRxTxLogFile(rxtxLogFile.getText());
            }
        });
        rxtxLogFile.setText(Preferences.getInstance().getRxTxLogFile());
        c.gridx += 1;
        c.weightx = 1;
        add(rxtxLogFile, c);

        JCheckBox mock = new JCheckBox(Resources.getString(getClass(), "mock"), Preferences.getInstance().isMock());

        mock.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                final boolean mock = e.getStateChange() == ItemEvent.SELECTED;

                commPorts.setEnabled(!mock);
                fireMockChanged(mock);
            }
        });
        c.gridwidth = 2;
        c.gridx = 0;
        c.gridy += 1;
        c.weightx = 0;
        c.weighty = 1;
        add(mock, c);

        model.addPreferencesModelListener(new PreferencesModelListener() {
            @Override
            public void commPortsChanged() {
                setCommPorts(commPorts);
            }
        });
    }

    public void addPreferencesViewListener(PreferencesViewListener l) {
        listeners.add(l);
    }

    private void fireCommPortChanged(CommPort commPort) {
        for (PreferencesViewListener l : listeners) {
            l.commPortChanged(commPort);
        }
    }

    private void fireMockChanged(boolean mock) {
        for (PreferencesViewListener l : listeners) {
            l.mockChanged(mock);
        }
    }

    private void setCommPorts(JComboBox commPorts) {
        commPorts.removeAllItems();

        String activeCommPortName = Preferences.getInstance().getCommPort();
        LOGGER.debug("The active comm port from preferences: {}", activeCommPortName);
        CommPort activeCommPort = null;
        for (CommPort commPort : model.getCommPorts()) {
            commPorts.addItem(commPort);
            if (activeCommPortName != null
                && activeCommPortName.equals(commPort.toString()/*getIdentifier().getName()*/)) {
                activeCommPort = commPort;
                //                commPorts.setSelectedItem(commPort);
                //                fireCommPortChanged(commPort);
            }
        }
        if (activeCommPort != null) {
            LOGGER.debug("Select the active commport: {}", activeCommPort);
            commPorts.setSelectedItem(activeCommPort);
            fireCommPortChanged(activeCommPort);
        }
        else {
            LOGGER.debug("Add empty item because no default active port is set.");
            CommPort commPort = new CommPort(null);
            commPorts.addItem(commPort);
            commPorts.setSelectedItem(commPort);
            fireCommPortChanged(commPort);
        }
    }
}
