package org.bidib.wizard.mvc.preferences.view.panel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.text.NumberFormat;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerDateModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.preferences.model.Preferences;
import org.bidib.wizard.mvc.preferences.view.listener.PreferencesViewListener;

public class TimePanel extends JPanel {
    private static final long serialVersionUID = -5927872456501821169L;

    private final Collection<PreferencesViewListener> listeners = new LinkedList<PreferencesViewListener>();

    public TimePanel() {
        setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Resources.getString(getClass(),
            "title")
            + ":"));
        setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 0;
        c.weighty = 0;
        c.insets = new Insets(5, 5, 5, 5);
        add(new JLabel(Resources.getString(getClass(), "timeFactor") + ":"), c);

        final JTextField timeFactor = new JFormattedTextField(NumberFormat.getNumberInstance());

        timeFactor.getDocument().addDocumentListener(new DocumentListener() {
            @Override
            public void changedUpdate(DocumentEvent e) {
                valueChanged();
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                valueChanged();
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                valueChanged();
            }

            private void valueChanged() {
                if (!timeFactor.getText().isEmpty()) {
                    try {
                        fireTimeFactorChanged(Integer.parseInt(timeFactor.getText()));
                    }
                    catch (NumberFormatException e) {
                    }
                }
            }
        });
        timeFactor.setText(String.valueOf(Preferences.getInstance().getTimeFactor()));
        c.gridx++;
        c.weightx = 1;
        add(timeFactor, c);

        c.anchor = GridBagConstraints.LINE_START;
        c.gridx = 0;
        c.gridy++;
        c.weightx = 0;
        add(new JLabel(Resources.getString(getClass(), "startTime") + ":"), c);

        final JSpinner startTime = new JSpinner(new SpinnerDateModel());

        startTime.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                JSpinner spinner = (JSpinner) e.getSource();

                fireStartTimeChanged((Date) spinner.getValue());
            }
        });
        startTime.setEditor(new JSpinner.DateEditor(startTime, "HH:mm"));
        startTime.setValue(Preferences.getInstance().getStartTime());
        c.gridx++;
        c.weightx = 1;
        add(startTime, c);

        // dummy label for layout reasons
        c.gridx = 0;
        c.gridy++;
        c.weightx = 0;
        c.weighty = 1;
        add(new JLabel(), c);
    }

    public void addPreferencesViewListener(PreferencesViewListener l) {
        listeners.add(l);
    }

    private void fireStartTimeChanged(Date startTime) {
        for (PreferencesViewListener l : listeners) {
            l.startTimeChanged(startTime);
        }
    }

    private void fireTimeFactorChanged(int timeFactor) {
        for (PreferencesViewListener l : listeners) {
            l.timeFactorChanged(timeFactor);
        }
    }
}
